#!/bin/bash

TEMPLATES=templates
TMP=/tmp/
TMPDIR=/tmp/$TEMPLATES
SINGLE=single
MULTI=multi
ALLHOSTDIR=allhost
DATAHOSTDIR=datahost
BUNDLE=hce-node-bundle
CFG=$BUNDLE/api/php/cfg
INI=$BUNDLE/api/php/ini
MANAGE=$BUNDLE/api/php/manage

NODESREG='^[1-9]$|^10$'
HOSTSREG='^[0-9]+$'
OCTET='(25[0-5]|2[0-4][0-9]|[01]?[0-9]?[0-9])'
IPREG='^\$OCTET\.\$OCTET\.\$OCTET\.\$OCTET$'
REPLICAS_POOL_ADMIN_PORTS_N=5530
REPLICAS_POOL_ADMIN_PORTS_M=5630

# FINCTIONS #
function validIp() {
    local  ip=$1
    local  stat=1
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]
    then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

function choiseSystem() {
    read -p "Single or Multihost system [SINGLE/multi]: " SYSTEM
    if [[ ! $SYSTEM ]]
    then
        echo single
    else
        if [[ $SYSTEM == single ]]
        then
            echo single
        elif [[ $SYSTEM == multi ]]
        then
            echo multi
        else
            exit 1
        fi
    fi
}

function tmpPath() {
    read -p "Path for created files (without slash in end): " TMPPATH
    if [[ ! $TMPPATH ]]
    then
        echo /tmp/
    else
        echo $TMPPATH/
    fi
}

function ipOfHosts() {
    read -p "IP for host № "$COUNTER" [127.0.0.1]: " IP
    if [[ ! $IP ]]
    then
        echo 127.0.0.1
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function ipOfRouter() {
    read -p "IP for router ["${IP[1]}"]: " IP
    if [[ ! $IP ]]
    then
        echo ${IP[1]}
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function ipOfManager() {
    read -p "IP for manager ["${IP[1]}"]: " IP
    if [[ ! $IP ]]
    then
        echo ${IP[1]}
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function numberOfHosts() {
    read -p "Number of hosts [2]-[...]: " HOSTS
    if [[ ! $HOSTS ]]
    then
        echo 2
    elif [[ $HOSTS =~ $HOSTSREG ]]
    then
        echo $HOSTS
    else
        exit 1
    fi
}

function numberOfNodes() {
    read -p "Number of nodes [1]-10: " NODES
    if [[ ! $NODES ]]
    then
        echo 1
    elif [[ $NODES =~ $NODESREG ]]
    then
        echo $NODES
    else
        exit 1
    fi
}

function typeOfManager() {
    read -p "Type of manager [n]/m: " MANAGER_TYPE
    if [[ ! $MANAGER_TYPE ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'n' ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'm' ]]
    then
        echo m
    else
        exit 1
    fi
}

function typeOfCluster() {
    read -p "Type of cluster [smanager]/rmanager/rmanager-rnd: " CLUSTER
    if [[ ! $CLUSTER ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'smanager' ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'rmanager' ]]
    then
        echo rmanager
    elif [[ $CLUSTER == 'rmanager-rnd' ]]
    then
        echo rmanager-rnd
    else
        exit 1
    fi
}

function replicasPoolAdminPorts () {
    # $1 - name of manager, $2 - count of nodes
    for (( COUNT=0; COUNT<$2; COUNT++ ))
    do
        PORT=$[$1+$COUNT]
        if [[ ! $PORTS ]]
        then
            PORTS=$PORT
        else
            PORTS=${PORTS}' '$PORT
        fi
    done
    echo $PORTS
}

function search () {
    # $1 - filename, where search
    # $2 - search string
    while read line
    do
        if [[ $line == $2 ]]
        then
            echo true
            exit 1
        fi
    done < $1
}
# END OF FINCTIONS #

# CHECK ARCHIVE WITH TEMPLATES #
if [[ ! -d $TEMPLATES ]]
then
    echo "Directory with templates not found. Aborting"
    exit
fi
# END OF CHECK ARCHIVE WITH TEMPLATES #

# GET VARIABLES #
clear
if [[ $# -ne 0 ]]
then
# FROM FILES #
    FILE=$1
    if [[ -a $FILE ]]
    then
        while read line
        do
            if [[ $(echo $line | grep \"HOSTS\") ]]
            then
                HOSTS=$(echo $line | awk -F\" '/"HOSTS"/ {print $4}')
            elif [[ $(echo $line | grep \"ROUTER_IP\") ]]
            then
                ROUTER_IP=$(echo $line | awk -F\" '/"ROUTER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"MANAGER_IP\") ]]
            then
                MANAGER_IP=$(echo $line | awk -F\" '/"MANAGER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"CLUSTER\") ]]
            then
                CLUSTER=$(echo $line | awk -F\" '/"CLUSTER"/ {print $4}')
            elif [[ $(echo $line | grep \"IP\") ]]
            then
                COUNTER=1
                IPS=$(echo $line | grep \"IP\" | awk -F[ '{print $2}' | awk -F] '{print $1}')
                for I in $(echo $IPS | tr "," "\n")
                do
                    IPTMP=$(echo $I | sed "s/\"//g")
                    IP[$COUNTER]=$IPTMP
                    unset IPTMP
                    let COUNTER=COUNTER+1
                done
            fi
        done < $FILE
    fi
else
# FROM STDIN #
    # GET OUT PATH #
    OUTDIR=$(tmpPath)

    # SINGLE OR MULTI SYSTEM #
    SYSTEM=$(choiseSystem)
    if [[ -z $SYSTEM ]]
    then
        echo "System not correct, select single or multi"
        exit 1
    fi
    if [[ $SYSTEM == single ]]
    then
    # SINGLE SYSTEM #
        NODES=$(numberOfNodes)
        if [[ -z $NODES ]]
        then
            echo "Number of nodes is not valid"
            exit 1
        fi
        MANAGER_TYPE=$(typeOfManager)
        if [[ -z $MANAGER_TYPE ]]
        then
            echo "Manager type is not valid"
            exit 1
        fi
    elif [[ $SYSTEM == multi ]]
    then
    # MULTI SYSTEM #
        COUNTER=1
        HOSTS=$(numberOfHosts)
        if [[ -z $HOSTS ]]
        then
            echo "Number of hosts is not valid"
            exit 1
        fi
        while [[  $COUNTER -le $HOSTS ]]
        do
            IPTMP=$(ipOfHosts)
            if [[ -z $IPTMP ]]
            then
                echo "IP address is not valid"
                exit 1
            else
                IP[$COUNTER]=$IPTMP
            fi
            NODESTMP=$(numberOfNodes)
            if [[ -z $NODESTMP ]]
            then
                echo "Number of nodes is not valid"
                exit 1
            else
                NODES[$COUNTER]=$NODESTMP
            fi
            MANAGER_TYPETMP=$(typeOfManager)
            if [[ -z $MANAGER_TYPETMP ]]
            then
                echo "Manager type is not valid"
                exit 1
            else
                MANAGER_TYPE[$COUNTER]=$MANAGER_TYPETMP
            fi
            let COUNTER=COUNTER+1
        done
        ROUTER_IP=$(ipOfRouter)
        if [[ -z $ROUTER_IP ]]
        then
            echo "IP address for router is not valid"
            exit 1
        fi
        MANAGER_IP=$(ipOfManager)
        if [[ -z $MANAGER_IP ]]
        then
            echo "IP address for manager is not valid"
            exit 1
        fi
        CLUSTER=$(typeOfCluster)
        if [[ -z $CLUSTER ]]
        then
            echo "Type of cluster is not valid"
            exit 1
        fi
    else
        exit 1
    fi
fi
# END OF GET VARIABLES #

# CREATE CONFIGS #
# DELETE OLD CONFIGS #
rm -rf $TMPDIR

if [[ $SYSTEM == single ]]
then
# SINGLE SYSTEM #
    N_NAME=localhost_n_cfg.sh
    M_NAME=localhost_m_cfg.sh
    mkdir -p $TMPDIR/$SINGLE/$CFG
    # CP current_cfg.sh TO TEMPLATES #
    if [[ -f ../cfg/current_cfg.sh ]]
    then
        cp ../cfg/current_cfg.sh $TMPDIR/$SINGLE/$CFG
    else
        cp $TEMPLATES/$SINGLE/$CFG/current_cfg.sh $TMPDIR/$SINGLE/$CFG/
    fi
    if [[ $MANAGER_TYPE == n ]]
    then
        # SEARCH IN current_cfg.sh DUBLE CONFIG #
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        # COPY N-CONFIG OT TEMPLATE DIRECTORY #
        cp $TEMPLATES/$SINGLE/$CFG/n_cfg.sh $TMPDIR/$SINGLE/$CFG/$N_NAME
        # PORTS OF NODES #
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N $NODES)
        sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/$SINGLE/$CFG/$N_NAME
    elif [[ $MANAGER_TYPE == m ]]
    then
        # SEARCH IN current_cfg.sh DUBLE CONFIG #
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        # COPY N-CONFIG OT TEMPLATE DIRECTORY #
        cp $TEMPLATES/$SINGLE/$CFG/m_cfg.sh $TMPDIR/$SINGLE/$CFG/$M_NAME
        # PORTS OF NODES #
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M $NODES)
        sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/$SINGLE/$CFG/$M_NAME
    else
        exit 1
    fi
# END OF SINGLE SYSTEM #

else
# MULTI SYSTEM #
    mkdir $TMPDIR
    for (( COUNT=1; COUNT<=$HOSTS; COUNT++ ))
    do
        mkdir -p $TMPDIR/${IP[$COUNT]}/$CFG
        mkdir -p $TMPDIR/${IP[$COUNT]}/$INI
        mkdir -p $TMPDIR/${IP[$COUNT]}/$MANAGE
        N_NAME=${IP[$COUNT]}_n_cfg.sh
        M_NAME=${IP[$COUNT]}_m_cfg.sh
        if [[ ${IP[$COUNT]} == $MANAGER_IP && ${IP[$COUNT]} == $ROUTER_IP ]]
        then
            # MANAGER + ROUTER + DATA HOST #
            # CP current_cfg.sh TO TEMPLATES #
            if [[ -f ../cfg/current_cfg.sh ]]
            then
                cp ../cfg/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            else
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            fi
            # MAKE node_pool1.ini #
            cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$INI/node_pool1.ini $TMPDIR/${IP[$COUNT]}/$INI/
            sed -i 's/_DATAIP_/'${IP[$COUNT]}'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini
            sed -i 's/_NOTIFICATIONIP_/'${IP[$COUNT]}'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini
            if [[ ${MANAGER_TYPE[$COUNT]} == n ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG OT TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/_RHOST1_/'$ROUTER_IP'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # MANAGE #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$MANAGE/properties_n.sh $TMPDIR/${IP[$COUNT]}/$MANAGE/

            elif [[ ${MANAGER_TYPE[$COUNT]} == m ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY m-CONFIG OT TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M ${NODES[$COUNT]})
                sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                sed -i 's/_RHOST1_/'$ROUTER_IP'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # MANAGE #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$MANAGE/properties_m.sh $TMPDIR/${IP[$COUNT]}/$MANAGE/
                sed -i 's/_CFG_NAME_/'$M_NAME'/g' $TMPDIR/${IP[$COUNT]}/$MANAGE/properties_m.sh
            else
                exit 1
            fi
        elif [[ ${IP[$COUNT]} == $MANAGER_IP && ${IP[$COUNT]} != $ROUTER_IP ]]
        then
            # MANAGER + DATA - ROUTER #
            echo "manager+data-router"
        elif [[ ${IP[$COUNT]} != $MANAGER_IP && ${IP[$COUNT]} == $ROUTER_IP ]]
        then
            # ROUTER + DATA - MANAGER #
            echo "manager+data-router"
        elif [[ ${IP[$COUNT]} != $MANAGER_IP && ${IP[$COUNT]} != $ROUTER_IP && $MANAGER_IP == $ROUTER_IP ]]
        then
            # ROUTER + MANAGER - DATA #
            # CP current_cfg.sh TO TEMPLATES #
            if [[ -f ../cfg/current_cfg.sh ]]
            then
                cp ../cfg/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            else
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            fi
            # MAKE node_pool1.ini #
            cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$INI/node_pool1.ini $TMPDIR/${IP[$COUNT]}/$INI/
            sed -i 's/_DATAIP_/'${IP[$COUNT]}'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini
            sed -i 's/_NOTIFICATIONIP_/'$MANAGER_IP'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini

            if [[ ${MANAGER_TYPE[$COUNT]} == n ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG OT TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
            elif [[ ${MANAGER_TYPE[$COUNT]} == m ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY m-CONFIG OT TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M ${NODES[$COUNT]})
                sed -i 's/_PORTLIST_/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
            else
                exit 1
            fi
        else
            # ALL IN SINGLE HOST #
            echo "all in single host"
        fi
    done
fi
# MV 2 OUTDIR #
if [[ $TMP != $OUTDIR ]]
then
    mkdir -p $OUTDIR
    mv $TMPDIR $OUTDIR
fi

echo Complete