#!/bin/bash

TEMPLATES=templates
TMP=/tmp/
TMPDIR=/tmp/$TEMPLATES
SINGLE=single
MULTI=multi
ALLHOSTDIR=allhost
RMHOST=router_manager
DATAHOSTDIR=datahost
BUNDLE=hce-node-bundle
CFG=$BUNDLE/api/php/cfg
INI=$BUNDLE/api/php/ini
MANAGE=$BUNDLE/api/php/manage

NODESREG='^[1-9]$|^10$'
HOSTSREG='^[0-9]+$'
OCTET='(25[0-5]|2[0-4][0-9]|[01]?[0-9]?[0-9])'
IPREG='^\$OCTET\.\$OCTET\.\$OCTET\.\$OCTET$'
REPLICAS_POOL_ADMIN_PORTS_N=5530
REPLICAS_POOL_ADMIN_PORTS_M=5630

# FINCTIONS #
function validIp() {
    local  ip=$1
    local  stat=1
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]
    then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

function tmpPath() {
    read -p "Path for created files (without slash in end): " TMPPATH
    if [[ ! $TMPPATH ]]
    then
        echo /tmp/
    else
        echo $TMPPATH/
    fi
}

function ipOfHosts() {
    read -p "IP for host № "$COUNTER" [127.0.0.1]: " IP
    if [[ ! $IP ]]
    then
        echo 127.0.0.1
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function ipOfRouter() {
    read -p "IP for router+manager [127.0.0.1]: " IP
    if [[ ! $IP ]]
    then
        echo 127.0.0.1
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function numberOfHosts() {
    read -p "Number of data hosts [1]-[...]: " HOSTS
    if [[ ! $HOSTS ]]
    then
        echo 1
    elif [[ $HOSTS =~ $HOSTSREG ]]
    then
        echo $HOSTS
    else
        exit 1
    fi
}

function numberOfNodes() {
    read -p "Number of nodes [1]-10: " NODES
    if [[ ! $NODES ]]
    then
        echo 1
    elif [[ $NODES =~ $NODESREG ]]
    then
        echo $NODES
    else
        exit 1
    fi
}

function typeOfManager() {
    read -p "Type of manager [n]/m: " MANAGER_TYPE
    if [[ ! $MANAGER_TYPE ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'n' ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'm' ]]
    then
        echo m
    else
        exit 1
    fi
}

function typeOfCluster() {
    read -p "Type of cluster [smanager]/rmanager/rmanager-rnd: " CLUSTER
    if [[ ! $CLUSTER ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'smanager' ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'rmanager' ]]
    then
        echo rmanager
    elif [[ $CLUSTER == 'rmanager-rnd' ]]
    then
        echo rmanager-rnd
    else
        exit 1
    fi
}

function replicasPoolAdminPorts () {
    # $1 - name of manager, $2 - count of nodes
    for (( COUNT=0; COUNT<$2; COUNT++ ))
    do
        PORT=$[$1+$COUNT]
        if [[ ! $PORTS ]]
        then
            PORTS=$PORT
        else
            PORTS=${PORTS}' '$PORT
        fi
    done
    echo $PORTS
}

function search () {
    # $1 - filename, where search
    # $2 - search string
    while read line
    do
        if [[ $line == $2 ]]
        then
            echo true
            exit 1
        fi
    done < $1
}
# END OF FINCTIONS #

# CHECK ARCHIVE WITH TEMPLATES #
if [[ ! -d $TEMPLATES ]]
then
    echo "Directory with templates not found. Aborting"
    exit
fi
# END OF CHECK ARCHIVE WITH TEMPLATES #

# GET VARIABLES #
clear
if [[ $# -ne 0 ]]
then
# FROM FILES #
    FILE=$1
    if [[ -a $FILE ]]
    then
        while read line
        do
            if [[ $(echo $line | grep \"HOSTS\") ]]
            then
                HOSTS=$(echo $line | awk -F\" '/"HOSTS"/ {print $4}')
            elif [[ $(echo $line | grep \"ROUTER_IP\") ]]
            then
                ROUTER_IP=$(echo $line | awk -F\" '/"ROUTER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"MANAGER_IP\") ]]
            then
                MANAGER_IP=$(echo $line | awk -F\" '/"MANAGER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"CLUSTER\") ]]
            then
                CLUSTER=$(echo $line | awk -F\" '/"CLUSTER"/ {print $4}')
            elif [[ $(echo $line | grep \"IP\") ]]
            then
                COUNTER=1
                IPS=$(echo $line | grep \"IP\" | awk -F[ '{print $2}' | awk -F] '{print $1}')
                for I in $(echo $IPS | tr "," "\n")
                do
                    IPTMP=$(echo $I | sed "s/\"//g")
                    IP[$COUNTER]=$IPTMP
                    unset IPTMP
                    let COUNTER=COUNTER+1
                done
            fi
        done < $FILE
    fi
else
# FROM STDIN #
    # GET OUT PATH #
    OUTDIR=$(tmpPath)

    ROUTER_IP=$(ipOfRouter)
    MANAGER_IP=$ROUTER_IP
    if [[ -z $ROUTER_IP ]]
    then
        echo "IP address for router+manager is not valid"
        exit 1
    fi

    COUNTER=1
    HOSTS=$(numberOfHosts)
    if [[ -z $HOSTS ]]
    then
        echo "Number of hosts is not valid"
        exit 1
    fi
    while [[  $COUNTER -le $HOSTS ]]
    do
        IPTMP=$(ipOfHosts)
        if [[ -z $IPTMP ]]
        then
            echo "IP address is not valid"
            exit 1
        else
            IP[$COUNTER]=$IPTMP
        fi
        NODESTMP=$(numberOfNodes)
        if [[ -z $NODESTMP ]]
        then
            echo "Number of nodes is not valid"
            exit 1
        else
            NODES[$COUNTER]=$NODESTMP
        fi
        MANAGER_TYPETMP=$(typeOfManager)
        if [[ -z $MANAGER_TYPETMP ]]
        then
            echo "Manager type is not valid"
            exit 1
        else
            MANAGER_TYPE[$COUNTER]=$MANAGER_TYPETMP
        fi
        let COUNTER=COUNTER+1
    done

    CLUSTER=$(typeOfCluster)
    if [[ -z $CLUSTER ]]
    then
        echo "Type of cluster is not valid"
        exit 1
    fi

fi
# END OF GET VARIABLES #

# CREATE CONFIGS #
# DELETE OLD CONFIGS #
rm -rf $TMPDIR

mkdir $TMPDIR

# MANAGER + ROUTER #

mkdir -p $TMPDIR/$ROUTER_IP/$CFG
mkdir -p $TMPDIR/$ROUTER_IP/$INI
mkdir -p $TMPDIR/$ROUTER_IP/$MANAGE

N_NAME=$ROUTER_IP'_n_cfg.sh'
M_NAME=$ROUTER_IP'_m_cfg.sh'

# CP current_cfg.sh AND n_cfg.sh AND m_cfg.sh TO TEMPLATES #
cp $TEMPLATES/$MULTI/$RMHOST/$CFG/current_cfg.sh $TMPDIR/$ROUTER_IP/$CFG/
cp $TEMPLATES/$MULTI/$RMHOST/$CFG/n_cfg.sh $TMPDIR/$ROUTER_IP/$CFG/$N_NAME
cp $TEMPLATES/$MULTI/$RMHOST/$CFG/m_cfg.sh $TMPDIR/$ROUTER_IP/$CFG/$M_NAME
# ADD TO current_cfg.sh #
sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/current_cfg.sh
sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/current_cfg.sh
# ADD RHOSTN AND RPORTN #
for (( COUNT=1; COUNT<=$HOSTS; COUNT++ ))
do
    sed -i 's/#_RHOST_#/'RHOST$COUNT="${IP[$COUNT]}\n#_RHOST_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/$N_NAME
    sed -i 's/#_RPORT_#/'"RPORT"$COUNT"=\"5530\"\n#_RPORT_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/$N_NAME
    sed -i 's/#_RHOST_#/'RHOST$COUNT="${IP[$COUNT]}\n#_RHOST_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/$M_NAME
    sed -i 's/#_RPORT_#/'"RPORT"$COUNT"=\"5530\"\n#_RPORT_#"'/g' $TMPDIR/$ROUTER_IP/$CFG/$M_NAME
done
# MANAGE #
cp $TEMPLATES/$MULTI/$RMHOST/$MANAGE/* $TMPDIR/$ROUTER_IP/$MANAGE/
# MAKE node_pool1.ini #
cp $TEMPLATES/$MULTI/$RMHOST/$INI/node_pool1.ini $TMPDIR/$ROUTER_IP/$INI/
sed -i 's/_DATAIP_/localhost/g' $TMPDIR/$ROUTER_IP/$INI/node_pool1.ini
sed -i 's/_NOTIFICATIONIP_/127.0.0.1/g' $TMPDIR/$ROUTER_IP/$INI/node_pool1.ini

# DATA HOSTS #
for (( COUNT=1; COUNT<=$HOSTS; COUNT++ ))
do
    mkdir -p $TMPDIR/${IP[$COUNT]}/$CFG
    mkdir -p $TMPDIR/${IP[$COUNT]}/$INI
    N_NAME=${IP[$COUNT]}_n_cfg.sh
    M_NAME=${IP[$COUNT]}_m_cfg.sh
    # CP current_cfg.sh TO TEMPLATES #
    cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
    if [[ ${MANAGER_TYPE[$COUNT]} == n ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
            # COPY N-CONFIG OT TEMPLATE DIRECTORY #
            cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
            # PORTS OF NODES #
            PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
            sed -i 's/__PORTLIST__/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
            sed -i 's/__MANAGER__/'$ROUTER_IP'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
        elif [[ ${MANAGER_TYPE[$COUNT]} == m ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
            # COPY m-CONFIG OT TEMPLATE DIRECTORY #
            cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
            # PORTS OF NODES #
            PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M ${NODES[$COUNT]})
            sed -i 's/__PORTLIST__/'"$PORTSLIST"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
            sed -i 's/__MANAGER__/'$ROUTER_IP'/g' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
        else
            exit 1
        fi
        # MAKE node_pool1.ini #
        cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$INI/node_pool1.ini $TMPDIR/${IP[$COUNT]}/$INI/
        sed -i 's/_DATAIP_/'${IP[$COUNT]}'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini
        sed -i 's/_NOTIFICATIONIP_/'$MANAGER_IP'/g' $TMPDIR/${IP[$COUNT]}/$INI/node_pool1.ini
done

# MV 2 OUTDIR #
if [[ $TMP != $OUTDIR ]]
then
    mkdir -p $OUTDIR
    mv $TMPDIR $OUTDIR
fi

echo Complete