'''
Created on Mar 28, 2014

@package: dtm
@author: scorp
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import json
import re
import sys
import os
import traceback
import urlparse
import Exceptions
# import urlnorm
import uritools
from scrapy.utils.url import canonicalize_url
from datetime import datetime
import logging
import app.Consts as APP_CONSTS


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

# #PropertiesValidator contains only 1 method - isValueIn, that find value in class(classType param)
# attributes that began with "prefix"
#
class PropertiesValidator(object):


  def __init__(self):
    pass


  @staticmethod
  def isValueIn(classType, prefix, value):
    retVal = False
    for localValue in classType.__dict__:
      if str(localValue).find(prefix) == 0:
        if value == getattr(classType, localValue, None):
          retVal = True
          break
    return retVal


# #getPath global function, finds and return value by path in json string or dict document
# dictionary - incoming dictionary (optional)
# jsonString - incoming json string (optional)
# path - incoming path to find
# method return valid value or raises exceptions -
# [ValueError] - bad jsodnString format
# [TypeError, KeyError, IndexError] - excpetions raised if path not found
# Warning!!! path don't checks by syntaxis
def getPath(dictionary, jsonString, path):
  if jsonString != None:
    dictionary = json.loads(jsonString)
  for i, p in re.findall(r'(\d+)|(\w+)', path):
    dictionary = dictionary[p or int(i)]
  return dictionary


# #Json serialization
#
class JsonSerializable(object):

  # #constructor
  # initialize task's fields
  #
  def __init__(self):
    pass

  @staticmethod
  def json_serial(obj):
    if isinstance(obj, datetime):
      return obj.isoformat()
    else:
      return obj.__dict__


  def toJSON(self):
    # return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    # Support custom serialization of datetime
    return json.dumps(self.__dict__, default=JsonSerializable.json_serial, sort_keys=True, indent=4)



class SQLExpression(object):
  def __init__(self, stringExpression=""):
    super(SQLExpression, self).__init__()
    if stringExpression is None:
      self.str = ""
    else:
      self.str = str(stringExpression)

  def __str__(self):
    return self.str



# #The PathMaker class
#
# This class purpose to use as static method container to split FS directory pattern on two parts
# @param string is a directory pattern
# @param subdirLen is a number of characters in sub-directory item
#
class PathMaker(object):

  SUBDIR_LEVEL1_LEN = 2
  SUBDIR_CHAR = "/"

  def __init__(self, string, subdirLen=SUBDIR_LEVEL1_LEN):
    super(PathMaker, self).__init__()
    self.string = string
    self.subdirLen = subdirLen

  def getDir(self):
    if len(self.string) > self.subdirLen:
      return self.string[:self.subdirLen] + self.SUBDIR_CHAR + self.string[self.subdirLen:]
    else:
      return self.string



# #The function to get traceback information string prepared for logging
#
# This function collects traceback information and creates sreing representation ready to log it
# @param linesNumberMax max number of traceback lines to include in to the collection, None - signs all
# @ret return string
#
def getTracebackInfo(linesNumberMax=None):
  ret = ""
  n = 0

  type_, value_, traceback_ = sys.exc_info()
  stack = traceback.format_tb(traceback_)
  del type_
  del value_
  for item in stack:
    ret = ret + "\n" + (str(item))
    n = n + 1
    if linesNumberMax != None and n == linesNumberMax:
      break

  return ret



# #The function to get a printable representation of an object for debugging
#
#
# @param obj The object to print
# @ret return string dump
#
'''
def varDump(obj):
  if '__dict__' in dir(obj):
    return str(vars(obj))
  else:
    return str(obj)
'''
def varDump(obj, stringify=True):
  newobj = obj
  if isinstance(obj, list):
    newobj = []
    for item in obj:
      newobj.append(varDump(item, False))
  elif isinstance(obj, tuple):
    temp = []
    for item in obj:
      temp.append(varDump(item, False))
    newobj = tuple(temp)
  elif isinstance(obj, set):
    temp = []
    for item in obj:
      temp.append(str(varDump(item, False)))
    newobj = set(temp)
  elif isinstance(obj, dict):
    newobj = {}
    for key, value in obj.items():
      newobj[str(varDump(key, False))] = varDump(value, False)
  # elif isinstance(obj, types.FunctionType):
  #  newobj = repr(obj)
  elif '__dict__' in dir(obj):
    newobj = obj.__dict__.copy()
    if ' object at ' in str(obj) and not '__type__' in newobj:
      newobj['__type__'] = str(obj).replace(" object at ", " #").replace("__main__.", "")
    for attr in newobj:
      newobj[attr] = varDump(newobj[attr], False)

  if stringify:
    return str(newobj)
  else:
    return newobj


def memUsage(point=""):
  import resource
  usage=resource.getrusage(resource.RUSAGE_SELF)
  return '''%s: mem=%s mb
      '''%(point,resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1000)



# #class UrlParser makes URL operation
#
class UrlParser(object):


  def __init__(self):
    pass


  # #The function check protocol and dimain name in the incoming url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def isValidURL(url):
    ret = False
    parseUrl = urlparse.urlparse(url)
    if parseUrl.scheme != None and parseUrl.netloc != None:
      ret = True
    return ret


  # #The function generates base url
  #
  # url - param, incoming url
  # @ret boolean
  @staticmethod
  def generateDomainUrl(url):
    ret = ""
    parseUrl = urlparse.urlparse(url)
    if UrlParser.isValidURL(url):
      ret = parseUrl.scheme + "://" + parseUrl.netloc
    else:
      raise Exceptions.UrlParseException("Empty protocol or domain name")
    return ret


# #class UrlNormalizator makes URL normalization
#
class UrlNormalizator(object):


  def __init__(self):
    pass


  # #The function normalize the incoming url according  RFC 3986
  #
  # url - param, incoming url
  # @ret normalized url
  @staticmethod
  def normalize(url):
    norm_url = url
    try:
      # logger.debug("input url: <%s>", url)
      # url.strip() removes trailing special characters like '\n'
      # without it urls inserted with '\n' symbol at the end
      # as result will be crawled only sites ended by '/' like 'html://www.foo.com/\n'
      # however full urls won't crawled like 'html:/www.foo.com/file.html\n'
      # norm_url = str(urlnorm.norm(url.strip()))
      # logger.debug("AAAAA: <%s>", norm_url)
      norm_url = str(uritools.urinormpath(url.strip()))
      #norm_url = str(canonicalize_url(url.strip()))
      # logger.debug("norm_url: <%s>", norm_url)
    # except urlnorm.InvalidUrl:
    #  logger.error("Normalization InvalidUrl")
    #  norm_url = ""
    except Exception as e:
      logger.error("Normalization error: " + str(e.message) + "\nURL: [" + url + "]")

    return norm_url

# ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
# # storePickleOnDisk
#
def storePickleOnDisk(input_pickled_object, env_path, file_name):
  if env_path in os.environ and os.environ[env_path] != "":
    logger.debug("os.environ[%s]: set to %s" % (env_path, os.environ[env_path]))
    open(os.environ[env_path] + file_name, "wb").write(input_pickled_object)
  else:
    logger.debug("os.environ[%s]: not set.", env_path)
