'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import logging
import os
import shutil
import tempfile
import traceback
import sys
import Constants
from dtm.EventObjects import GeneralResponse  # pylint: disable=unused-import
import dc.EventObjects

logger = logging.getLogger(Constants.LOGGER_NAME)

#template for key value file name
KEY_VALUE_FILE_NAME_TEMPLATE = "%s.db"
KEY_VALUE_FIELDS_FILE_NAME_TEMPLATE = "%s_fields.db"

##class implemented all logic necessary to process SiteCleanUp request
#
class SiteCleanUpTask(object):


  ##constructor
  #
  #@param keyValueStorageDir path to keyValue storage work dir
  #@param rawDataDir path to raw data dir
  def __init__(self, keyValueStorageDir, rawDataDir):
    self.keyValueStorageDir = keyValueStorageDir
    self.rawDataDir = rawDataDir
    self.errorCode = 0
    self.errorMessage = ""


  ##make all necessary actions to cleanup al site data
  #
  #@param siteCleanup instance of SiteCleanup object
  #@param queryCallback function for quieries execution
  #@return generalResponse instance of GeneralResponse object
  def process(self, siteCleanup, queryCallback):
    ret = GeneralResponse()
    self.cleanUpMysqlStorage(siteCleanup, queryCallback)
    self.cleanUpKeyValueDBStorage(siteCleanup, KEY_VALUE_FILE_NAME_TEMPLATE)
    self.cleanUpKeyValueDBStorage(siteCleanup, KEY_VALUE_FIELDS_FILE_NAME_TEMPLATE)
    self.cleanUpRawDataStorage(siteCleanup)
    self.cleanUpMysqlSiteTable(siteCleanup, queryCallback)
    ret.errorCode = self.errorCode
    ret.statuses.append(ret.errorCode)
    ret.errorMessage = self.errorMessage
    return ret


  ##cleanup all site data from mysql db
  #
  #@param siteCleanup instance of SiteCleanup object
  #@param queryCallback function for quieries execution
  def cleanUpMysqlStorage(self, siteCleanup, queryCallback):
    SQL_TRUNCATE_QUERY_TEMPLATE = "TRUNCATE TABLE `%s`"
    query = SQL_TRUNCATE_QUERY_TEMPLATE % (Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteCleanup.id)
    queryCallback(query, Constants.SECONDARY_DB_ID)


  ##sets empty values in `sites` table for some fields
  #
  #@param siteCleanup instance of SiteCleanup object
  #@param queryCallback function for quieries execution
  def cleanUpMysqlSiteTable(self, siteCleanup, queryCallback):
    CLEAR_SITE_RECORS_SQL = ("UPDATE `sites` SET TcDate=NULL, Resources=0, Iterations=0, State=%s, " +
    "ErrorMask=0, Errors=0, Contents=0, CollectedURLs=0 WHERE id = '%s'")
    query = CLEAR_SITE_RECORS_SQL % (str(dc.EventObjects.Site.STATE_ACTIVE), siteCleanup.id)
    queryCallback(query, Constants.PRIMARY_DB_ID)


  ##cleanup all site data from keyvalue db
  #
  #@param siteCleanup instance of SiteCleanup object
  def cleanUpKeyValueDBStorage(self, siteCleanup, filesSuffix):
    try:
      delFile = self.keyValueStorageDir + "/" + (filesSuffix % siteCleanup.id)
      logger.debug(">>> Del file = %s", str(delFile))
      os.remove(delFile)
    except OSError as err:
      type_, value_, traceback_ = sys.exc_info()  # pylint: disable=unused-variable
      stack = traceback.format_tb(traceback_)
      logger.debug(str(err.filename) + " " + str(err.strerror))
      logger.debug(str(stack.pop()))
      logger.debug(">>> [cleanUpKeyValueDBStorage] CURRENT DIR " + str(os.getcwd()))
#      self.errorCode = 1
#      self.errorMessage = (">>> cleanUpKeyValueDBStorage Error")


  ##cleanup all site data from raw data storage
  #
  #@param siteCleanup instance of SiteCleanup object
  def cleanUpRawDataStorage(self, siteCleanup):
    try:
      tmpDirName = self.rawDataDir + "/" + os.path.basename(tempfile.NamedTemporaryFile().name)
      originDirName = self.rawDataDir + "/" + siteCleanup.id
      logger.debug(">>> originDir = %s", str(originDirName))
      os.rename(originDirName, tmpDirName)
      shutil.rmtree(tmpDirName)
    except Exception as err:
      type_, value_, traceback_ = sys.exc_info()  # pylint: disable=W0612
      stack = traceback.format_tb(traceback_)
      logger.debug(str(err.filename) + " " + str(err.strerror))
      logger.debug(str(stack.pop()))
      logger.debug(">>> [cleanUpRawDataStorage] CURRENT DIR " + str(os.getcwd()))
#      self.errorCode = 2
#      self.errorMessage = (">>> cleanUpRawDataStorage Error")
