'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import logging
import MySQLdb.cursors
import MySQLdb as mdb
import dc.EventObjects as dc_event
import Constants
from contextlib import closing
from dc.Constants import EVENT_TYPES, DRCESyncTasksCover
from SiteTask import SiteTask
from SiteCleanUpTask import SiteCleanUpTask
from SiteDeleteTask import SiteDeleteTask
from SiteUpdateTask import SiteUpdateTask
from SiteStatusTask import SiteStatusTask
from SiteFindTask import SiteFindTask
from URLNewTask import URLNewTask
from URLStatusTask import URLStatusTask
from URLUpdateTask import URLUpdateTask
from URLFetchTask import URLFetchTask
from URLCleanupTask import URLCleanUpTask
from URLDeleteTask import URLDeleteTask
from URLContentTask import URLContentTask
from SQLCustomTask import SQLCustomTask
from Constants import LOGGER_NAME

logger = logging.getLogger(LOGGER_NAME)

# #dict to check event
eventCheckDict = dict({EVENT_TYPES.SITE_NEW: dc_event.Site,
                     EVENT_TYPES.SITE_UPDATE: dc_event.SiteUpdate,
                     EVENT_TYPES.SITE_STATUS: dc_event.SiteStatus,
                     EVENT_TYPES.SITE_DELETE: dc_event.SiteDelete,
                     EVENT_TYPES.SITE_CLEANUP: dc_event.SiteCleanup,
                     EVENT_TYPES.SITE_FIND: dc_event.SiteFind,
                     EVENT_TYPES.URL_NEW: list,
                     EVENT_TYPES.URL_UPDATE: list,
                     EVENT_TYPES.URL_STATUS: list,
                     EVENT_TYPES.URL_DELETE: list,
                     EVENT_TYPES.URL_FETCH: list,
                     EVENT_TYPES.URL_CLEANUP: list,
                     EVENT_TYPES.URL_CONTENT: list,
                     EVENT_TYPES.SQL_CUSTOM: dc_event.CustomRequest})

# #sql query which checks existence of a table
CHECK_TABLE_SQL_TEMPLATE = " SELECT COUNT(*) FROM sites WHERE Id = '%s'"

# @todo move to apropriate place
TASK_DUBLICATE_ERR = 2020
TASK_DUBLICATE_ERR_MSG = "Dublicate site"


# LOGGER_NAME = "db_task"

class TasksManager(object):
  # Configuration settings options names
  DB_HOST = "db_host"
  DB_PORT = "db_port"
  DB_USER = "db_user"
  DB_PWD = "db_pwd"
  PRIMARY_DB = "primary_db"
  SECONDARY_DB = "secondary_db"
  SITE_TEMPLATES = "dc_site_template"
  KEY_VALUE_STORAGE_DIR = "key_value_storage_dir"
  KEY_VALUE_DEFAULT_FILE = "key_value_default_file"
  RAW_DATA_DIR = "raw_data_dir"

  # #constructor
  #
  # @param configParser config parser object
  def __init__(self, configParser):
    className = self.__class__.__name__
    # Get configuration settings
    dbHost = configParser.get(className, self.DB_HOST)
    dbPort = int(configParser.get(className, self.DB_PORT))
    dbUser = configParser.get(className, self.DB_USER)
    dbPWD = configParser.get(className, self.DB_PWD)
    dbPrimary = configParser.get(className, self.PRIMARY_DB)
    dbSecondary = configParser.get(className, self.SECONDARY_DB)
    self.dbConnections = {}
    self.dbNames = {}

    self.dcSiteTemplate = configParser.get(className, self.SITE_TEMPLATES)
    self.keyValueStorageDir = configParser.get(className, self.KEY_VALUE_STORAGE_DIR)
    self.rawDataDir = configParser.get(className, self.RAW_DATA_DIR)
    self.keyValueDefaultFile = configParser.get(className, self.KEY_VALUE_DEFAULT_FILE)
    self.dbConnections[Constants.PRIMARY_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbPrimary, dbPort)
    self.dbNames[dbPrimary] = Constants.PRIMARY_DB_ID
    self.dbConnections[Constants.SECONDARY_DB_ID] = mdb.connect(dbHost, dbUser, dbPWD, dbSecondary, dbPort)
    self.dbNames[dbSecondary] = Constants.SECONDARY_DB_ID
    self.SQLErrorCode = Constants.EXIT_CODE_OK


  # #destructor
  #
  def __del__(self):
    for dbConnect in self.dbConnections.values():
      if dbConnect is not None:
        dbConnect.close()


  # #process input event
  #
  # @param drceSyncTasksCover instance of DRCESyncTasksCover
  # @return generalResponse instance of GeneralResponse object
  def process(self, drceSyncTasksCover):
#    print(type(dc_event.SiteStatus("")))

    self.checkInputData(drceSyncTasksCover)
    responseObject = None
    responseEvent = None
    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_DELETE:
      siteDeleteTask = SiteDeleteTask(self.keyValueStorageDir, self.rawDataDir)
      responseObject = siteDeleteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_DELETE_RESPONSE
      self.SQLErrorCode = Constants.EXIT_CODE_OK

    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_NEW:
      siteTask = SiteTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir)
      responseObject = siteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_NEW_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_UPDATE:
      siteUpdateTask = SiteUpdateTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir)
      responseObject = siteUpdateTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_UPDATE_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_CLEANUP:
      siteCleanUpTask = SiteCleanUpTask(self.keyValueStorageDir, self.rawDataDir)
      responseObject = siteCleanUpTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_CLEANUP_RESPONSE
      self.SQLErrorCode = Constants.EXIT_CODE_OK

    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_STATUS:
      siteStatusTask = SiteStatusTask()
      responseObject = siteStatusTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_STATUS_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_NEW:
      urlNewTask = URLNewTask()
      urlNewTask.siteTask = SiteTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir)
      responseObject = urlNewTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_NEW_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_STATUS:
      urlStatusTask = URLStatusTask()
      responseObject = urlStatusTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_STATUS_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_UPDATE:
      urlUpdateTask = URLUpdateTask()
      responseObject = urlUpdateTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_UPDATE_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_FETCH:
      urlFetchTask = URLFetchTask()
      responseObject = urlFetchTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_FETCH_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_CONTENT:
      urlContentTask = URLContentTask(self.keyValueStorageDir, self.rawDataDir)
      responseObject = urlContentTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_CONTENT_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_DELETE:
      urlDeleteTask = URLDeleteTask(self.keyValueStorageDir, self.rawDataDir)
      responseObject = urlDeleteTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_DELETE_RESPONSE

    if drceSyncTasksCover.eventType == EVENT_TYPES.URL_CLEANUP:
      urlCleanUpTask = URLCleanUpTask(self.keyValueStorageDir, self.rawDataDir)
      responseObject = urlCleanUpTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.URL_CLEANUP_RESPONSE

    # find site by root url and return list of siteId's with root urls
    if drceSyncTasksCover.eventType == EVENT_TYPES.SITE_FIND:
      siteFindTask = SiteFindTask(self.dcSiteTemplate, self.keyValueDefaultFile, self.keyValueStorageDir)
      responseObject = siteFindTask.process(drceSyncTasksCover.eventObject, self.executeQuery)
      responseEvent = EVENT_TYPES.SITE_FIND_RESPONSE
      
    if drceSyncTasksCover.eventType == EVENT_TYPES.SQL_CUSTOM:
      sqlCustomTask = SQLCustomTask()
      responseObject = sqlCustomTask.process(drceSyncTasksCover.eventObject, self.executeQuery, self.backDBResolve)
      responseEvent = EVENT_TYPES.SQL_CUSTOM_RESPONSE

    return self.createDRCESyncTasksCover(responseEvent, responseObject)


  # #executeQuery common entry point for SQL execution
  #
  # @param query query for execution
  # @param dbConnectionName MySQL connection Id
  # @executeType executeType - return set type
  # @returns fetching data or None
  def executeQuery(self, query, dbConnectionName, executeType=Constants.EXEC_INDEX):
    ret = None
    if executeType == Constants.EXEC_INDEX:
      ret = self.executeQueryByIndex(query, dbConnectionName)
    elif executeType == Constants.EXEC_NAME:
      ret = self.executeQueryByName(query, dbConnectionName)
    return ret


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQueryByIndex(self, query, dbConnectionName):
    ret = None
    dbConnection = None
    if dbConnectionName in self.dbConnections:
      dbConnection = self.dbConnections[dbConnectionName]
    if dbConnection:
      try:
        with closing(dbConnection.cursor()) as cursor:
          logger.info(query)
          cursor.execute(query)
          dbConnection.commit()
          ret = cursor.fetchall()
      except mdb.Error as err:
        dbConnection.rollback()
        errString = "%s %s message = %s" % (err.args[0], err.args[1], str(err.message))
        logger.error(errString)
        self.SQLErrorCode = Constants.EXIT_CODE_MYSQL_ERROR
#        sys.stderr.write(query)
    return ret


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQueryByName(self, query, dbConnectionName):
    ret = None
    dbConnection = None
    if dbConnectionName in self.dbConnections:
      dbConnection = self.dbConnections[dbConnectionName]
    if dbConnection:
      try:
        with closing(dbConnection.cursor(MySQLdb.cursors.DictCursor)) as cursor:
          logger.info(query)
          cursor.execute(query)
          dbConnection.commit()
          ret = cursor.fetchall()
      except mdb.Error as err:
        dbConnection.rollback()
        errString = "%s %s message = %s" % (err.args[0], err.args[1], str(err.message))
        logger.error(errString)
        self.SQLErrorCode = Constants.EXIT_CODE_MYSQL_ERROR
    return ret


  # #check  conformity data in input DRCESyncTasksCover object
  #
  # @param drceSyncTasksCover instance of DRCESyncTasksCover
  def checkInputData(self, drceSyncTasksCover):
    for inputType in eventCheckDict:
#      logger.info("eventCheckDict: "+ str(vars(eventCheckDict)))
#      logger.info("inputType: "+ str(vars(inputType)))
#      logger.info("drceSyncTasksCover.eventType: "+ str(drceSyncTasksCover.eventType))
      if drceSyncTasksCover.eventType == inputType and \
      isinstance(drceSyncTasksCover.eventObject, eventCheckDict[inputType]):
        return True
    raise Exception("Incorrect types of  input data")


  # #helper function to build response as DRCESyncTasksCover object
  #
  # @param eventType type of response event
  # @param eventObject instance of returning object
  def createDRCESyncTasksCover(self, eventType, eventObject):
    return DRCESyncTasksCover(eventType, eventObject)
  

  # #gets dbName and returns internar db index
  #
  # @param dbName db name
  def backDBResolve(self, dbName):
    ret = None
    if dbName in self.dbNames:
      ret = self.dbNames[dbName]
    return ret
    
