'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import logging  # pylint: disable=unused-import
import hashlib
import os
import sqlite3
import traceback
import sys
import Constants
import shutil
import MySQLdb
import FieldRecalculator
from app.Utils import PathMaker
from BaseTask import BaseTask
from dtm.EventObjects import GeneralResponse

logger = logging.getLogger(Constants.LOGGER_NAME)


##process urlCleanUp event
class URLCleanUpTask(BaseTask):


  ##constructor
  #
  #@param keyValueStorageDir path to keyValue storage work dir
  #@param rawDataDir path to raw data dir
  def __init__(self, keyValueStorageDir, rawDataDir):
    super(URLCleanUpTask, self).__init__()
    self.keyValueStorageDir = keyValueStorageDir
    self.rawDataDir = rawDataDir
    self.recalculator = FieldRecalculator.FieldRecalculator()


  ##make all necessary actions to delete urls data from db
  #
  #@param urlDelete list of URLDelete objects
  #@param queryCallback function for queries execution
  #@return generalResponse instance of GeneralResponse object
  def process(self, urlCleanups, queryCallback):
    generalResponse = GeneralResponse()
    for urlCleanup in urlCleanups:
      #@todo add more complex case
      if urlCleanup.siteId == "":
        urlCleanup.siteId = "0"
      if self.isSiteExist(urlCleanup.siteId, queryCallback):
        try:
          localUrls = []
          if urlCleanup.url is None:
            localUrls = self.extractUrlByCriterions(urlCleanup.siteId, True, urlCleanup.criterions, queryCallback)
          else:
            localUrls.append(urlCleanup.url)
          for localUrl in localUrls:
            urlCleanup.url = localUrl
            self.deleteFromKeyValue(urlCleanup)
            self.deleteFromRawStorage(urlCleanup)
            self.updateMysqlDB(urlCleanup, queryCallback)
          generalResponse.statuses.append(True)
          self.recalculator.siteResourcesRecalculate(urlCleanup.siteId, queryCallback)
          self.recalculator.siteContentsRecalculate(urlCleanup.siteId, queryCallback)
        except Exception:
          generalResponse.statuses.append(False)
          type_, value_, traceback_ = sys.exc_info()  # pylint: disable=unused-variable
          stack = traceback.format_tb(traceback_)
          logger.error(str(stack.pop()))

    return generalResponse


  #@extractUrlByCriterions method makes sql request. using criterions and returns result
  #
  #@param siteId - incoming siteId for SQL request addition cause
  #@param criterions - incoming criterions for SQL request
  #@return urls list (fetched from criterions sql)
  def extractUrlByCriterions(self, siteId, calcMD5, criterions, queryCallback):
    retUrls = []
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % siteId
    if calcMD5:
      SQLUrlExtractor = "SELECT `URL` FROM `%s`" % tableName
    else:
      SQLUrlExtractor = "SELECT `URLMd5` FROM `%s`" % tableName
    query = SQLUrlExtractor + self.generateCriterionSQL(criterions)
    res = queryCallback(query, Constants.SECONDARY_DB_ID)
    if hasattr(res, '__iter__'):
      logger.debug(">>> Select URL len(res) = " + str(len(res)))
      for row in res:
        retUrls.append(row[0])
    return retUrls


  #@todo urldelete - make base class
  ##delete data from key value db
  #
  #@param urlDelete instance of URLDelete object
  #@param queryCallback function for queries execution
  def deleteFromKeyValue(self, urlCleanup):
    #@todo try block + check table name
    dbName = Constants.KEY_VALUE_FILE_NAME_TEMPLATE % urlCleanup.siteId
    dbFileName = self.keyValueStorageDir + "/" + dbName
    if os.path.exists(dbFileName):
      dbConnect = sqlite3.connect(dbFileName)  # pylint: disable-all
      dbConnect.text_factory = str
      cursor = dbConnect.cursor()
      cursor.execute("DELETE from `articles` where Id = '%s'" % urlCleanup.url)
      dbConnect.commit()


  ##delete data from raw storage
  #
  #@param urlDelete instance of URLDelete object
  #@param queryCallback function for queries execution
  def deleteFromRawStorage(self, urlCleanup, calcMD5=True):
    if calcMD5:
      urlMd5 = hashlib.md5(urlCleanup.url).hexdigest()
    else:
      urlMd5 = urlCleanup.url
    dataDir = self.rawDataDir + "/" + urlCleanup.siteId + "/" + PathMaker(urlMd5).getDir()
    logger.debug(">>> CLEANUP DIR = " + str(dataDir))
    if os.path.isdir(dataDir):
      try:
        shutil.rmtree(dataDir)
      except OSError as ex:
        logger.error(">>> [%s] Dir delete error - MSG [%s]", dataDir, str(ex.message))


  ##update data in mysql db
  #
  #@param urlDelete instance of URLDelete object
  #@param queryCallback function for queries execution
  def updateMysqlDB(self, urlCleanup, queryCallback):
    UPDATE_SQL_TEMPLATE = "UPDATE `%s` SET state = %s, status = %s WHERE `URL` = '%s'"
    #@todo check later
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % urlCleanup.siteId
    query = UPDATE_SQL_TEMPLATE % (tableName, urlCleanup.state, urlCleanup.status,
                                   MySQLdb.escape_string(urlCleanup.url))
    queryCallback(query, Constants.SECONDARY_DB_ID)
