'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import logging  # pylint: disable=unused-import
import hashlib
import Constants
import dc.EventObjects
import FieldRecalculator
from BaseTask import BaseTask
from dtm.EventObjects import GeneralResponse
from URLCleanupTask import URLCleanUpTask

logger = logging.getLogger(Constants.LOGGER_NAME)

##process urlDelete event
class URLDeleteTask(BaseTask):


  ##constructor
  #
  #@param keyValueStorageDir path to keyValue storage work dir
  #@param rawDataDir path to raw data dir
  def __init__(self, keyValueStorageDir, rawDataDir):
    super(URLDeleteTask, self).__init__()
    self.uRLCleanUpTask = URLCleanUpTask(keyValueStorageDir, rawDataDir)
    self.recalculator = FieldRecalculator.FieldRecalculator()


  ##make all necessary actions to delete urls data from db
  #
  #@param urlDelete list of URLDelete objects
  #@param queryCallback function for queries execution
  #@return generalResponse instance of GeneralResponse object
  def process(self, urlDeletes, queryCallback):
    generalResponse = GeneralResponse()
    for urlDelete in urlDeletes:
      #@todo add more complex case
      if urlDelete.siteId == "":
        urlDelete.siteId = "0"
      if self.isSiteExist(urlDelete.siteId, queryCallback):
        try:
          localUrls = []
          if urlDelete.url is None:
            isUrlExtract = False
            if urlDelete.urlType == dc.EventObjects.URLStatus.URL_TYPE_URL:
              isUrlExtract = True
            localUrls = self.uRLCleanUpTask.extractUrlByCriterions(urlDelete.siteId, isUrlExtract,
                                                                   urlDelete.criterions, queryCallback)
          else:
            localUrls.append(urlDelete.url)
          logger.debug(">>> localUrls size = " + str(len(localUrls)))
          for localUrl in localUrls:
            try:
              urlDelete.url = localUrl
              self.uRLCleanUpTask.deleteFromKeyValue(urlDelete)
              if urlDelete.urlType == dc.EventObjects.URLStatus.URL_TYPE_URL:
                self.uRLCleanUpTask.deleteFromRawStorage(urlDelete, True)
              else:
                self.uRLCleanUpTask.deleteFromRawStorage(urlDelete, False)
              self.updateMysqlDB(urlDelete, queryCallback)
            except Exception as ex:
              logger.debug(">>> Some Type Exception [LOOP] = " + str(type(ex)) + " " + str(ex))
          generalResponse.statuses.append(True)
          self.recalculator.siteResourcesRecalculate(urlDelete.siteId, queryCallback)
          self.recalculator.siteContentsRecalculate(urlDelete.siteId, queryCallback)
          self.recalculator.updateCollectedURLs(urlDelete.siteId, queryCallback)
        except Exception as ex:
          logger.debug(">>> Some Type Exception = " + str(type(ex)) + " " + str(ex))
          generalResponse.statuses.append(False)
      else:
        generalResponse.statuses.append(False)

    return generalResponse


  ##update data in mysql db
  #
  #@param urlDelete instance of URLDelete object
  #@param queryCallback function for queries execution
  def updateMysqlDB(self, urlDelete, queryCallback):
    if urlDelete.urlType == dc.EventObjects.URLStatus.URL_TYPE_URL:
      localMd5 = hashlib.md5(urlDelete.url).hexdigest()
    else:
      localMd5 = urlDelete.url
    SQL_CLAUSE = ("`URLMd5` = '%s'" % localMd5)
    UPDATE_SQL_TEMPLATE = "DELETE FROM `%s` WHERE %s"
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % urlDelete.siteId
    query = UPDATE_SQL_TEMPLATE % (tableName, SQL_CLAUSE)
    queryCallback(query, Constants.SECONDARY_DB_ID)
