'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

from BaseTask import BaseTask
from dtm.EventObjects import GeneralResponse
from Constants import LOGGER_NAME
import Constants
import dc.EventObjects
import app.Utils
import hashlib
import app.Exceptions
import logging
import FieldRecalculator

logger = logging.getLogger(LOGGER_NAME)

##process list of URLs
class URLNewTask(BaseTask):

  ##constructor
  #
  def __init__(self):
    super(URLNewTask, self).__init__()
    self.siteTask = None
    self.recalculator = FieldRecalculator.FieldRecalculator()


  #Memethod creates new site, using SiteTask class
  #
  #@param initUrl base site url
  #@param queryCallback function for queries execution
  def newSiteCreate(self, initUrl, queryCallback):
    if self.siteTask is None:
      raise Exception(">>> URLNew.siteTask object is None!")
    localSiteObj = dc.EventObjects.Site(initUrl)
    self.siteTask.process(localSiteObj, queryCallback)


  #Memethod makes operation with "site" database table
  #
  #@param urlObject instance of URL object
  #@param queryCallback function for queries execution
  def fillSiteRelatedFields(self, urlObj, queryCallback):
    ret = False
    SITE_EXTRACT_SQL_QUERY = "SELECT `RequestDelay`, `HTTPTimeout`, `URLType` FROM `sites` WHERE id = '%s'"
    res = queryCallback(SITE_EXTRACT_SQL_QUERY % urlObj.siteId, Constants.PRIMARY_DB_ID)
    if hasattr(res, '__iter__') and len(res) > 0:
      ret = True
      if urlObj.requestDelay is None:
        urlObj.requestDelay = res[0][0]
      if urlObj.httpTimeout is None:
        urlObj.httpTimeout = res[0][1]
      if urlObj.type is None:
        urlObj.type = res[0][2]
    return ret


  #Try to extract siteId from MySQL database, based on incoming url field
  #
  #@param url - URL.URL field of incoming object
  #@param queryCallback function for queries execution
  def resolveSiteIdByURL(self, url, queryCallback):
    ret = None
    query = Constants.SELECT_SITE_ID_BY_URL % url
    res = queryCallback(query, Constants.PRIMARY_DB_ID)
    if hasattr(res, '__iter__') and len(res) > 0:
      ret = res[0][0]
      logger.debug(">>> Site_Id By URL = %s", str(ret))
    return ret


  #Memethod makes operation with "site" database table
  #
  #@param urlObject instance of URL object
  #@param queryCallback function for queries execution
  def siteTableOperation(self, urlObj, queryCallback):
    if urlObj.siteSelect == dc.EventObjects.URL.SITE_SELECT_TYPE_EXPLICIT:
      if urlObj.siteId == "" or self.isSiteExist(urlObj.siteId, queryCallback) == False:
        urlObj.siteId = "0"
    elif urlObj.siteSelect == dc.EventObjects.URL.SITE_SELECT_TYPE_AUTO:
      canonicUrl = None
      localSiteId = None
      try:
        canonicUrl = app.Utils.UrlParser.generateDomainUrl(urlObj.url)
        localSiteId = hashlib.md5(canonicUrl).hexdigest()
        if self.isSiteExist(localSiteId, queryCallback):
          urlObj.siteId = localSiteId
          self.fillSiteRelatedFields(urlObj, queryCallback)
        else:
          self.newSiteCreate(canonicUrl, queryCallback)
          urlObj.siteId = localSiteId
      except app.Exceptions.UrlParseException:
        pass
    elif urlObj.siteSelect == dc.EventObjects.URL.SITE_SELECT_TYPE_QUALIFY_URL:
      canonicUrl = app.Utils.UrlParser.generateDomainUrl(urlObj.url)
      localSiteId = hashlib.md5(canonicUrl).hexdigest()
      if self.isSiteExist(localSiteId, queryCallback) == False:
        urlObj.siteId = "0"
    else:
      raise Exception(">>> urlObj.siteSelect field has wrong value - %s" % str(urlObj.siteSelect))


  ##make all necessary actions to add new URLs into mysql db
  #
  #@param urls list of URL objects
  #@param queryCallback function for queries execution
  #@return generalResponse instance of GeneralResponse object
  def process(self, urls, queryCallback):
    ret = GeneralResponse()
    status = 1
    isRelatedSite = False
    for url in urls:
      #@todo add more complex case
      if url.siteId is None and url.siteSelect != dc.EventObjects.URL.SITE_SELECT_TYPE_EXPLICIT:
        url.siteId = self.resolveSiteIdByURL(url.url, queryCallback)
      if self.isSiteExist(url.siteId, queryCallback):
        isRelatedSite = self.fillSiteRelatedFields(url, queryCallback)
      if not isRelatedSite:
        self.siteTableOperation(url, queryCallback)
      if url.siteId is not None and url.siteId != "":
        if self.selectURL(url, queryCallback) == False:
          self.addURL(url, queryCallback)
          status = 0
        else:
          status = 2
        self.recalculator.siteResourcesRecalculate(url.siteId, queryCallback)
        self.recalculator.siteContentsRecalculate(url.siteId, queryCallback)
        self.recalculator.updateCollectedURLs(url.siteId, queryCallback)
      ret.statuses.append(status)
    return ret


  ##update urls
  #
  #@param urlObject instance of URL object
  #@param queryCallback function for queries execution
  def selectURL(self, urlObject, queryCallback):
    ret = False
    LOCAL_URL_CHECK_QUERY = "SELECT COUNT(*) FROM `urls_%s` WHERE URL = '%s'"
    query = LOCAL_URL_CHECK_QUERY % (urlObject.siteId, urlObject.url)
    res = queryCallback(query, Constants.SECONDARY_DB_ID)
    if hasattr(res, '__iter__') and len(res) > 0 and len(res[0]) > 0 and res[0][0] > 0:
      ret = True
    return ret


  ##inserts new url
  #
  #@param urlObject instance of URL object
  #@param queryCallback function for queries execution
  def addURL(self, urlObject, queryCallback):
    ret = False
    LOCAL_QUERY_TEMPLATE = "INSERT IGNORE INTO `%s` SET %s"
    fields, values = Constants.getFieldsValuesTuple(urlObject, Constants.URLTableDict)
    fieldValueString = Constants.createFieldsValuesString(fields, values)
    if fieldValueString is not None and fieldValueString != "":
      query = LOCAL_QUERY_TEMPLATE % ((Constants.DC_URLS_TABLE_NAME_TEMPLATE % urlObject.siteId), fieldValueString)
      queryCallback(query, Constants.SECONDARY_DB_ID)
      ret = True
    return ret
