# coding: utf-8  # pylint: disable-all

"""@package docstring
 @file Scraper.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import re
import icu
import sys
import time
import json
import datetime
import copy
import hashlib
from lepl.apps.rfc3696 import HttpUrl
import ast
import pickle
import sqlite3  # as sqlite3
from email.Utils import formatdate
from dateutil.parser import *
from urlparse import urljoin
from urlparse import urlparse
import logging.config
import ConfigParser
import MySQLdb as mdb
import MySQLdb.cursors
from contextlib import closing
import app.Utils as Utils  # pylint: disable=F0401
from app.Utils import varDump

from scrapy.selector import Selector
from cement.core import foundation
from newspaper_extractor import NewspaperExtractor
from goose_extractor import GooseExtractor
from scrapy_extractor import ScrapyExtractor
from ml_extractor import MLExtractor
from scraper_utils import encode
from scraper_resource import Resource as Res
from scraper_result import Result as Result
from dc_processor.ScraperResponse import ScraperResponse
import app.Utils as Utils  # pylint: disable=F0401
from dc.EventObjects import SiteFilter
import app.Consts as APP_CONSTS
from scrapy.selector import Selector
import dc_processor.Constants as CONSTS


APP_NAME = "scraper"

MSG_ERROR_LOAD_CONFIG = "Error loading config file. Exciting."
MSG_ERROR_LOAD_LOG_CONFIG_FILE = "Error loading logging config file. Exiting."
MSG_ERROR_LOAD_EXTRACTORS = "Error load extractors "
MSG_ERROR_TEMPLATE_EXTRACTION = "Error template extraction "
MSG_ERROR_DYNAMIC_EXTRACTION = "Error dynamic extraction "
MSG_ERROR_LOAD_DB_BACKEND = "Error load db backend"
MSG_ERROR_LOAD_OPTIONS = "Error load options"
MSG_INFO_PREPARE_CONTENT = "Prepare content: "
MSG_ERROR_ADJUST_PR = "Error adjust partial references. "
MSG_ERROR_ADJUST_PUBDATE = "PUBDATE_ERROR "
MSG_ERROR_ADJUST_TITLE = "Can't adjust title. "

EXIT_SUCCESS = 0
EXIT_FAILURE = 1

SQLITE_TIMEOUT = 30

ENV_SCRAPER_STORE_PATH = "ENV_SCRAPER_STORE_PATH"
# #Scraper
#
#
class Scraper(foundation.CementApp):


  # Mandatory
  class Meta(object):
    label = APP_NAME
    def __init__(self):
      pass


  # #constructor
  # initialize default fields
  def __init__(self):
    # call base class __init__ method
    foundation.CementApp.__init__(self)
    self.exit_code = EXIT_SUCCESS
    self.itr = None
    self.extractor = None
    self._extractors = None
    self.input_data = None
    self.logger = None
    self.config_db_dir = None
    self.articles_tbl = None
    self.sqliteTimeout = SQLITE_TIMEOUT
    self.DBConnector = None
    self.db_dc_scraper = None
    self.db_dc_scraper_db_connect = None
    self.properties = {}
    self.algorithm_name = None
    self.pubdate = None
    self.timezone = None


  # #setup
  # setup application
  def setup(self):
    # call base class setup method
    foundation.CementApp.setup(self)
    self.args.add_argument('-c', '--config', action='store', metavar='config_file', help='config ini-file')


  # #run
  # run application
  def run(self):
    # call base class run method
    foundation.CementApp.run(self)

    # config section
    self.loadConfig()

    # load logger config file
    self.loadLogConfigFile()

    # load sqlite db backend
    # self.loadSqliteDBBackend()

    # mysql
    self.loadDBBackend()
  
    # mysql
    self.loadPropertiesFromDB()
    
    # options
    self.loadOptions()

    # init extractor's list
    self.loadExtractors()
  


  # #adjust partial references
  # adjust partial references
  #
  def checkDOMElement(self, elem):
    ret = False
    valid_http_url = HttpUrl()
    if re.search('<', elem):
        self.logger.debug("Media tag contain DOM element: %s", elem)
        ret = True
    return ret


  # #adjust partial references
  # adjust partial references
  #
  def adjustPartialReferences(self, response):
    try:
      url = None
      if self.input_data.template and "link" in self.input_data.template:
        self.logger.debug("url type: %s", str(type(response.tags["link"]["data"])))
        if type(response.tags["link"]["data"]) == str or type(response.tags["link"]["data"]) == unicode:
          url = response.tags["link"]["data"]
        else:
          url = response.tags["link"]["data"][0]
      else:
        url = self.input_data.url
      if self.input_data.template and "media" in self.input_data.template:
        self.logger.debug("resource has template with media tag. Try to adjust media.")
        if type(response.tags["media"]) == str and response.tags["media"] == "": return
        self.logger.debug("response.tags['media']: " + str(response.tags["media"]))
        self.logger.info("media tag in response: <<%s>>" % str(response.tags["media"]["data"]))
        self.logger.info("link tag in response: <<%s>>" % str(url))
        res = []

        filter_patterns, filter_types = [], []
        if self.input_data.filters:
          filter_types = [filter_item["Type"] for filter_item in self.input_data.filters]
          filter_patterns = [re.compile(filter_item["Pattern"]) for filter_item in self.input_data.filters]
        self.logger.debug("filter: %s" % (str(self.input_data.filters)))
        for media in response.tags["media"]["data"]:
          self.logger.info("Media link: <<%s>>", media)
          # we have dom like <img src="/images/photo_spacer.png" class="lazy" data-original="/photo/20140815/20140815-OYT1I50006-N.jpg" alt="">
          # instead pure url
          if self.checkDOMElement(media):
            res.append(media)
            break
          media = urljoin(url, media)
          valid_http_url = HttpUrl()
          for filter_type, filter_pattern in zip(filter_types, filter_patterns):
            match = filter_pattern.match(media)
            if filter_type == SiteFilter.TYPE_EXCLUDE and match:
              break
            if filter_type == SiteFilter.TYPE_INCLUDE and match:
              res.append(media)
              break
          else:
            self.logger.debug("media: %s", media)
            self.logger.debug("url: %s", url)
            if valid_http_url(media) and Scraper.parseHost(media) == Scraper.parseHost(url):
              res.append(media)

        # If media tag after adjusting is empty remove it from response
        if not len(res):
          self.logger.debug("media tag is empty. Remove media tag from response.")
          del response.tags["media"]
        else:
          self.logger.debug("media tag is adjusted. Copy media tag to response.")
          response.tags["media"]["data"] = res
        # End code block removing empty media tag
      else:
        self.logger.debug("resource hasn't template with media tag. Don't need adjust media.")
    except Exception as err:
      # self.exit_code = EXIT_FAILURE
      self.logger.debug(MSG_ERROR_ADJUST_PR + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      # raise


  # adjustLinkURL
  #
  def adjustTitle(self, response):
    try:
      if self.input_data.template and "title" in self.input_data.template:
        self.logger.debug("resource has template with title tag. Try to adjust title.")
        self.logger.debug("response.tags['title']: " + str(response.tags["title"]))
        if not self.extractor:
          self.extractor = self._extractors[2]
        if type(response.tags["title"]) == str:
          self.logger.info("response has not have title tag")
          sel = Selector(text=self.input_data.raw_content)
          title = sel.xpath("//title/text()").extract()
          self.extractor.addTag(result=response, tag_name="title", tag_value=title)
        self.logger.info("TYPE response.tags['title']['data']" + str(type(response.tags["title"]["data"])))
      else:
        self.logger.debug("resource hasn't template with title tag. Don't need adjust title.")
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_TITLE + err.message)


  # adjustLinkURL
  #
  def adjustLinkURL(self, response):
    flag = False
    try:
      if self.input_data.template and "link" in self.input_data.template:
        self.logger.debug("resource has template with link tag. Try to adjust link.")
        self.logger.debug("response.tags['link']: " + str(response.tags["link"]))
        self.logger.debug("self.extractor: %s", str(self.extractor))
        flag = True
        if self.extractor:
          self.logger.debug("Extractor exists")
          if type(response.tags["link"]) == str:
            self.logger.info("response has not have link tag")
            self.extractor.addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
          # bypass
          else:
            response.tags["link"]["data"] = self.input_data.url
        else:
          self._extractors[2].addTag(result=response, tag_name="link", tag_value=[self.input_data.url])
        self.logger.info("TYPE response.tags['link']['data']" + str(type(response.tags["link"]["data"])))
      else:
        self.logger.debug("resource hasn't template with link tag. Don't need adjust link.")
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_PR + err.message)
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
    return flag


  # adjustLinkURL
  #
  def adjustJapaneseDate(self, raw_pubdate):
    # convert raw_pubdate in format like
    # 8月20日 20:01
    # 9月3日 11時41分
    # （8月20日 紙面から）
    # 2014年8月7日
    # 2014年8月20日 夕刊
    # 2014年8月21日 朝刊>
    # 2014/8/14付
    # （2014/08/16-14:46）
    # 2014/6/3 18:53 (2014/6/3 20:13更新)
    # 2014.08.20 Wed posted at 17:52 JST
    # 2014.08.01 Fri posted at 12:36 JST
    # 2014年 08月 20日 14:34 JST
    # 2014年08月20日 19時46分
    # 2014年8月29日16時30分
    # 8/20 21:00 更新
    # 11月２３日
    # to
    # 2014年8月7日 20:01
    # check if 日 exist in date
    self.logger.debug("pubdate has to be converted: <<%s>>", raw_pubdate)
    self.logger.debug("pubdate type: <<%s>>", str(type(raw_pubdate)))
    #self.logger.debug("pubdate charcode: <<%s>>", str(icu.CharsetDetector(raw_pubdate).detect().getName()))
    #raw_pubdate = raw_pubdate.decode("utf_8")
    #self.logger.debug("pubdate has to be converted: <<%s>>", raw_pubdate)
    #self.logger.debug("pubdate type: <<%s>>", str(type(raw_pubdate)))
    raw_pubdate = raw_pubdate.strip(" \r\t\n)（）\xe6\x9b\xb4\xe6\x96\xb0\xef\xbc\x89\xe5\x88\x86\xe4\xbb\x98JST\xe7\xb4\x99\xe9\x9d\xa2\xe3\x81\x8b\xe3\x82　\xe5\xa4\x95\xe5\x88\x8a\xe6\x9c\x9d\xe5\x88\x8a\xe3\x80\x80\xe5\x88\x86")
    # 2014.08.20 Wed posted at 17:52 JST
    # if "Wed posted at" in raw_pubdate:
    #  date_items = raw_pubdate.replace(" Wed posted at ", " ").replace(".", " ").split()
    #  raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + "\xe6\x97\xa5 " + date_items[3]
    if "\xe6\x97\xa5" in raw_pubdate:
      raw_pubdate = raw_pubdate.replace("\xe6\x97\xa5", "\xe6\x97\xa5 ")
      # check if date contain year
      # 2014年08月20日 19時46分
      if "\xe5\xb9\xb4" in raw_pubdate and "\xe6\x99\x82" in raw_pubdate:
        raw_pubdate = raw_pubdate.replace("\xe6\x99\x82", ":")
      elif "\xe5\xb9\xb4" in raw_pubdate and not ":" in raw_pubdate:
        raw_pubdate = raw_pubdate + " 00:00"
      # 2014年08月19日 11:53
      elif "\xe5\xb9\xb4" in raw_pubdate and ":" in raw_pubdate and raw_pubdate.count(" ") == 1:
        pass
      # 2014年 08月 20日 14:34 JST
      elif "\xe5\xb9\xb4" in raw_pubdate and ":" in raw_pubdate and raw_pubdate.count(" ") > 1:
        date_items = raw_pubdate.split()
        if isinstance(date_items, list) and len(date_items) == 2:
          raw_pubdate = date_items[0] + " " + date_items[1]
        else:
          raw_pubdate = date_items[0] + date_items[1] + date_items[2] + " " + date_items[3]
      elif "\xe5\xb9\xb4" in raw_pubdate:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + raw_pubdate
    # 2014/8/14付
    if raw_pubdate.count("/") == 2:
      # date_items = raw_pubdate.split("/")
      # （2014/08/16-14:46）
      date_items = re.split("/|-", raw_pubdate)
      if isinstance(date_items, list) and len(date_items) == 4:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + "\xe6\x97\xa5 " + date_items[3]
      else:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + "\xe6\x97\xa5 00:00"
    # 8月20日 紙面から
    if not "\xe5\xb9\xb4" in raw_pubdate and not "/" in raw_pubdate:
      date_items = raw_pubdate.replace("\xe6\x9c\x88", " ").replace("\xe6\x97\xa5", " ").replace("\xe6\x99\x82", " ").split()
      if isinstance(date_items, list) and len(date_items) == 4:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + "\xe6\x97\xa5 " + date_items[2] + ":" + date_items[3]
      else:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + "\xe6\x97\xa5 00:00"
    # 8/20 21:00 更新
    if not "\xe5\xb9\xb4" in raw_pubdate and "/" in raw_pubdate:
      date_items = raw_pubdate.replace("/", " ").split()
      # 2014/6/3 18:53 (2014/6/3 20:13更新)
      if len(date_items) > 4:
        raw_pubdate = date_items[0] + "\xe5\xb9\xb4" + date_items[1] + "\xe6\x9c\x88" + date_items[2] + "\xe6\x97\xa5 " + date_items[3]
      else:
        raw_pubdate = str(time.gmtime().tm_year) + "\xe5\xb9\xb4" + date_items[0] + "\xe6\x9c\x88" + date_items[1] + "\xe6\x97\xa5 " + date_items[2]
    self.logger.debug("pubdate converted is: <<%s>>", raw_pubdate)
    return raw_pubdate


  # getTimeZone
  #
  def getTimeZone(self, raw_pubdate):
    # check if site has timezone rule
    if self.input_data.timezone is not None:
      self.timezone = self.input_data.timezone
      self.logger.debug("Resource has timezone properties. Timezone is: %s" % self.input_data.timezone)
    else:
      self.logger.debug("pubdate hasn't timezone")
    for tz in CONSTS.TIMEZONE_LIST:
      # check if date has timezone
      if tz in raw_pubdate:
        self.timezone = tz
        self.logger.debug("pubdate <<%s>> has timezone: <<%s>>" % (raw_pubdate, self.timezone))
        break
    self.logger.debug("Timezone is: %s" % self.timezone)


  # adjustTimeZone
  #
  def adjustTimeZone(self, pubdate):
    if self.timezone is not None:
      pubdate = re.sub("[+-]\d{4}", self.timezone, pubdate)
    return pubdate


  # convertPubDateToRFC2822
  #
  def convertPubDateToRFC2822(self, raw_pubdate):

    #replace all unicode digits to decimal (e.g. u'\uff13' to '3')
    import unicodedata
    for i in range(0, len(raw_pubdate)):
      if raw_pubdate[i].isdigit():
        raw_pubdate = re.sub(raw_pubdate[i], str(unicodedata.digit(raw_pubdate[i])), raw_pubdate)
    raw_pubdate = raw_pubdate.encode("utf_8")

    self.getTimeZone(raw_pubdate)
    import calendar
    try:
      pubdate_parts = raw_pubdate.split(",")
      if len(pubdate_parts)>1:
        raw_pubdate = pubdate_parts[0]
      raw_pubdate = raw_pubdate.replace("posted at", "")
      raw_pubdate = raw_pubdate.replace("Updated:", "")
      raw_pubdate = re.sub("\(1/\d{1}ページ\)", "", raw_pubdate)
      raw_pubdate = re.sub("平成26", "2014", raw_pubdate)
      raw_pubdate = re.sub("\(木\)", "", raw_pubdate)
      raw_pubdate = re.sub(" 年 ", "年", raw_pubdate)
      raw_pubdate = re.sub("@", "", raw_pubdate)
      parsed_time_candidate_str = float(calendar.timegm(parse(raw_pubdate).timetuple()))
      self.logger.debug("pubdate in seconds: %s", str(parsed_time_candidate_str))
    except ValueError:
      raw_pubdate = self.adjustJapaneseDate(raw_pubdate)
      parsed_time_candidate_str = float(calendar.timegm(time.strptime(raw_pubdate, u"%Y\xe5\xb9\xb4%m\xe6\x9c\x88%d\xe6\x97\xa5 %H:%M")))
      self.logger.debug("parsed_time_candidate_str: %s", str(parsed_time_candidate_str))
    time_str = formatdate(parsed_time_candidate_str)
    self.logger.debug("pubdate converted is: %s", str(time_str))
    time_str = self.adjustTimeZone(time_str)
    self.logger.debug("pubdate adjusted timezone is: %s", str(time_str))
    return time_str


  # adjustPubDate
  #
  def adjustPubDate(self, response):
    # adjust date in format
    # Fri, 21 Nov 1997 09:55:06 -0600
    flag = False
    pubdate = None
    try:
      if (self.input_data.template and (CONSTS.TAG_PUB_DATE in self.input_data.template)) \
        or not self.input_data.template:
        self.logger.debug("resource has template with pubdate tag or dynamic scraping. Try to adjust pubdate.")
        #extractor = self._extractors[2]
        extractor = self.getExtractorByName("ScrapyExtractor")
        # self.logger.debug("response.tags: %s", str(response.tags))
        if CONSTS.TAG_PUB_DATE in response.tags and response.tags[CONSTS.TAG_PUB_DATE] != "":
          self.logger.debug("response has 'pubdate' attr. pubdate is: %s", response.tags[CONSTS.TAG_PUB_DATE]["data"])
          # if any(i.isdigit() for i in response.tags[CONSTS.TAG_PUB_DATE]["data"]):
          response.tags[CONSTS.TAG_PUB_DATE]["data"] = self.convertPubDateToRFC2822(response.tags[CONSTS.TAG_PUB_DATE]["data"])
          flag = True
        else:
          self.logger.debug("response hasn't pubdate. Try to extract one.")
          sel = Selector(text=self.input_data.raw_content)
          for path in extractor.template[CONSTS.TAG_PUB_DATE]:
            time_candidate_str = sel.xpath(path).extract()
            if type(time_candidate_str) == list and len(time_candidate_str) > 0 \
              and time_candidate_str[0].strip(' \t\n\r') != "" \
              and flag == False \
              and any(i.isdigit() for i in time_candidate_str):
              self.logger.debug("Time candidate extracted by xpath: <<%s>> is: <<%s>>" % (path, time_candidate_str))
              time_candidate_str_stripped = time_candidate_str[0].strip(' \t\n\r')
              self.logger.debug("Time candidate: %s", time_candidate_str_stripped)
              # time_candidate_str_encoded = time_candidate_str_stripped.encode("utf-8", "ignore")
              # self.logger.debug("Extracted date: %s", time_candidate_str_encoded)
              converted_pubdate = self.convertPubDateToRFC2822(time_candidate_str_stripped)
              flag = extractor.addTag(result=response, tag_name=CONSTS.TAG_PUB_DATE, tag_value=converted_pubdate, xpath=path)
              self.logger.debug("tagsCount after: %s", response.tagsCount)
              self.logger.debug("response adjusted pubdate: %s", response.tags["pubdate"])
        if flag == False:
          self.logger.debug("Extracting pubdate has failed. Remove pubdate from response.")
          del response.tags["pubdate"]
          #converted_pubdate = self.convertPubDateToRFC2822(str(self.input_data.lastModified))
          #extractor.addTag(result=response, tag_name=CONSTS.TAG_PUB_DATE, tag_value=converted_pubdate)
      else:
        self.logger.debug("resource hasn't template with pubdate tag. Don't need adjust pubdate.")
      # self.logger.debug("pubdate: " + str(pubdate))
    except Exception as err:
      self.logger.debug(MSG_ERROR_ADJUST_PUBDATE + err.message)
      #tbi = Utils.getTracebackInfo()
      #self.logger.error(str(vars(err)) + "\n" + tbi)
      if CONSTS.TAG_PUB_DATE in response.tags:
        self.logger.debug("Extracting pubdate has failed but pubdate tag still exist in response. Remove pubdate from response.")
        del response.tags[CONSTS.TAG_PUB_DATE]
    # pubdate has been extracted from content properly
    if flag:
      from dateutil import parser
      pubdate = parser.parse(response.tags["pubdate"]["data"])
      # pubdate = datetime.datetime.strptime(response.tags["pubdate"]["data"], '%a, %d %b %Y %H:%M:%S %:z')
      self.logger.debug("pubdate: " + str(pubdate))
    return pubdate


  # #main content processing
  # main content processing
  #
  def process(self):
    # info input data
    self.logger.info("input_data.url: " + str(self.input_data.url))
    self.logger.info("input_data.urlId: " + str(self.input_data.urlId))
    self.logger.info("input_data.siteId: " + str(self.input_data.siteId))
    self.logger.info("input_data.filters: " + str(self.input_data.filters))
    # self.logger.info("input_data.template: " + str(self.input_data.template))
    self.logger.info("input_data.content len: %s", str(len(self.input_data.raw_content)))
    self.logger.info("input_data.lastModified data: %s", str(self.input_data.lastModified))
    self.logger.info("timezone: %s", str(self.input_data.timezone))

    # get iterator to ranked list of extractors
    self.itr = iter(sorted(self._extractors, key=lambda extractor: extractor.rank, reverse=True))

    # if for given resource exist template collection
    # then perform template extraction
    # else perform dynamic extraction
    if  self.input_data.siteId=="d57f144e7b26c9976769ea94f18b9064" or \
      self.input_data.siteId=="1fe592caf03fd50c5f065c30f82b13bb":
      response = self.searchEngineScraping()
    elif self.input_data.template:
      response = self.templateExtraction()
    else:
      response = self.dynamicExtraction()
    # put extracted article to rhe db
    # self.logger.debug("Response: " + response.get())
    # adjust partial references in media tag
    if self.input_data.siteId not in ["d57f144e7b26c9976769ea94f18b9064", "1fe592caf03fd50c5f065c30f82b13bb"]:
      self.adjustTitle(response)
      self.adjustLinkURL(response)
      self.adjustPartialReferences(response)
      self.pubdate = self.adjustPubDate(response)
      self.getProcessedContent(response)
      self.tagsCount = response.tagsCount
      self.tagsMask = response.tagsMask
      self.putArticleToDB(response)
      self.logger.debug("self.tagsCount: %s", self.tagsCount)
      self.logger.debug("self.tagsMasks: %s", self.tagsMask)
    return response


  def templateExtraction(self):
    self.extractor = ScrapyExtractor(self.config, self.input_data.template)
    # create empty result
    reslt = Result(self.config, self.input_data.urlId)
    # self.logger.info("Empty result: " + str(vars(reslt)))
    # self.logger.info("Raw content for template extracting: " + str(self.input_data.raw_content))
    sel = Selector(text=self.input_data.raw_content)
    # self.article_tags = [tag for tag in self.config.options("article_tags")]
    # self.logger.info("template: " + self.input_data.template)
    # self.logger.info("template type: " + str(type(self.input_data.template)))
    template = ast.literal_eval(self.input_data.template)
    # self.logger.info("template_data: " + str(template))
    for key in template:
      # self.logger.info("Template key: " + key)
      # self.logger.info("Template value: " + str(template[key]))
      # [extractor.addTag(result=reslt, tag_name=key, tag_value=sel.xpath(path).extract()) for path in template[key]]
      for path in template[key]:
        self.extractor.addTag(result=reslt, tag_name=key, tag_value=sel.xpath(path).extract(), xpath=path)
    return reslt


  def dynamicExtraction(self):
    # self.logger.info("self.input_data.urlId: " + self.input_data.urlId)
    # get resource as dictionary
    # resource_set = self.getResourceFromDB(resId)
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    # self.logger.info("raw_content: " + str(self.input_data.raw_content))
    resource = Res(resource_set)

    # create empty result
    result = Result(self.config, self.input_data.urlId)

    # get list of empty tags from result
    empty_tags = result.getEmptyTags()
    self.logger.info("get list of empty tags from result: " + str(empty_tags))

    # get best matching extractor
    self.extractor = self.getNextBestExtractor()
    self.logger.info("get best matching extractor: " + str(self.extractor))

    # search engine parsing
    if not resource.site_id:
      return self.extractor.extractTags(resource, result)

    # main loooop
    while len(empty_tags) and self.extractor:
      result = self.extractor.extractTags(resource, result)
      # print result.tags
      empty_tags = result.getEmptyTags()
      self.logger.info("get list of empty tags from result: " + str(empty_tags))
      filled_tags = result.getFilledTags()
      self.logger.info("get list of filled_tags from result: " + str(filled_tags))
      self.extractor = self.getNextBestExtractor()
      self.logger.info("get best matching extractor: " + str(self.extractor))
    return result


  def searchEngineScraping(self):
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    resource = Res(resource_set)
    result = Result(self.config, self.input_data.urlId)

    self.extractor = ScrapyExtractor(self.config, self.input_data.template)
    return self.extractor.extractTags(resource, result)


  def getNextBestExtractor(self):
    # return extractor with highest rank
    try:
      extractor = next(self.itr)
    except StopIteration:
      extractor = None
    return extractor


  def getProcessedContent(self, result):
    # Add md5 hash of content
    #self.logger.debug("result: %s" % varDump(result))
    self.processedContent = result.get()


  def putArticleToDB(self, result):
    # get appropriate db name, depending on siteId
    if len(self.input_data.siteId):
      db_name = self.config_db_dir + "/" + self.input_data.siteId + ".db"
    else:
      db_name = self.config_db_dir + "/0.db"
    self.logger.info("db_name: " + db_name)
    connector = None
    try:
      # put parsed resource to the db
      connector = sqlite3.connect(db_name, timeout=self.sqliteTimeout)  # @UndefinedVariable
      with connector:
        cur = connector.cursor()
        # query = "CREATE TABLE IF NOT EXISTS \
        #        %s(id VARCHAR(32) PRIMARY KEY UNIQUE, data TEXT, CDate DATETIME DEFAULT CURRENT_TIMESTAMP)" \
        #        % (self.articles_tbl)
        # cur.execute(query)
        # query = "INSERT OR REPLACE INTO `%s`(`id`,`data`, CDate) VALUES('%s', '%s')"\
        #        % (self.articles_tbl, result.resId, encode(result.get()))
        query = "INSERT OR REPLACE INTO `" \
                + self.articles_tbl + \
                "`(`id`,`data`, CDate) VALUES('" \
                + result.resId + \
                "', '" \
                + encode(result.get()) + \
                "', strftime('%Y-%m-%d %H:%M:%f', 'now', 'localtime'))"
        # self.logger.info("result: " + result.get())
        # self.logger.info("query: " + query)
        cur.execute(query)
    except Exception as err:
      # Connection objects can be used as context managers that automatically commit or rollback transactions.
      # In the event of an exception, the transaction is rolled back; otherwise, the transaction is committed:
      # connector.rollback()
      self.logger.error("%s " % (err.message))
      raise


  # #load extractors
  #
  def loadExtractors(self):
    try:
      # init extractor's list
      #self._extractors = [
      #    nex(self.config),
      #    gex(self.config),
      #    sex(self.config),
      #    MLExtractor(self.config)
      #]
      # modules
      self.algorithm_name = self.properties[CONSTS.ALGORITHM_KEY][CONSTS.ALGORITHM_NAME_KEY]
      modules = self.properties[CONSTS.MODULES_KEY][self.algorithm_name]

      self.logger.debug("Algorithm name: <%s>" % (self.algorithm_name))
      self.logger.debug("Modules: %s" % modules)

      self._extractors = [self.createModule(module) for module in modules]
    except Exception as err:
      print MSG_ERROR_LOAD_EXTRACTORS + err.message
      raise


  # #process batch
  # the main processing of the batch object
  def processBatch(self):
    try:
      # read pickled batch object from stdin and unpickle it
      input_pickled_object = sys.stdin.read()
      scraper_in_data = pickle.loads(input_pickled_object)
      self.input_data = scraper_in_data
      Utils.storePickleOnDisk(input_pickled_object, ENV_SCRAPER_STORE_PATH, "scraper.in." + str(self.input_data.urlId))
      # self.logger.info("input scraper object: " + str(vars(scraper_in_data)))
      # TODO main processing over every url from list of urls in the batch object
      tmp = sys.stdout
      sys.stdout = open("/dev/null", "wb")
      
      if self.algorithm_name == CONSTS.PROCESS_ALGORITHM_REGULAR:
        self.logger.debug("Process: %s" % self.algorithm_name)
        response = self.process()
      elif self.algorithm_name == CONSTS.PROCESS_ALGORITHM_TRAINING:
        self.logger.debug("Process: %s" % self.algorithm_name)
        response = self.processTraining()
      elif self.algorithm_name == CONSTS.PROCESS_ALGORITHM_CONCURRENCY:
        self.logger.debug("Process: %s" % self.algorithm_name)
        response = self.processConcurrency()

      # send response to the stdout
      sys.stdout = tmp
      if self.input_data.siteId in ["d57f144e7b26c9976769ea94f18b9064", "1fe592caf03fd50c5f065c30f82b13bb"]:
        print response.get()
      else:
        scraperResponse = ScraperResponse(self.tagsCount, self.tagsMask, self.pubdate, self.processedContent)
        output_pickled_object = pickle.dumps(scraperResponse)
        Utils.storePickleOnDisk(output_pickled_object, ENV_SCRAPER_STORE_PATH, "scraper.out." + str(self.input_data.urlId))
        print output_pickled_object
      sys.stdout.flush()
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      self.logger.error("Scraper process batch error: " + err.message)
      self.exit_code = EXIT_FAILURE


  # #load config from file
  # load from cli argument or default config file
  def loadConfig(self):
    try:
      self.config = ConfigParser.ConfigParser()
      self.config.optionxform = str
      if self.pargs.config:
        self.config.read(self.pargs.config)
      else:
        self.config.read(APP_NAME)
    except:
      print MSG_ERROR_LOAD_CONFIG
      raise


  # #load logging
  # load logging configuration (log file, log level, filters)
  #
  def loadLogConfigFile(self):
    try:
      log_conf_file = self.config.get("Application", "log")
      logging.config.fileConfig(log_conf_file)
      # logging.basicConfig()
      # self.logger = logging.getLogger(APP_NAME)
      # Logger initialization
      self.logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

    except:
      print MSG_ERROR_LOAD_LOG_CONFIG_FILE
      raise


  # #load mandatory options
  # load mandatory options
  #
  def loadOptions(self):
    try:
      self.config_db_dir = self.config.get(self.__class__.__name__, "config_db_dir")
      self.articles_tbl = self.config.get("sqlite", "articles_tbl")
      self.sqliteTimeout = self.config.getint("sqlite", "timeout")
    except:
      print MSG_ERROR_LOAD_OPTIONS
      raise


  ## processTraining
  #
  def processTraining(self):
    #TODO: add processTraining method implementation
    #stub
    self.modules[CONSTS.TRAINING_QUEUE] = [
      nex(self.config),
      gex(self.config)
    ]
    self.modules[CONSTS.TRAINED_QUEUE] = [
      sex(self.config)
    ]

    self._extractors = self.modules[CONSTS.TRAINING_QUEUE]
    self.response = self.dynamicExtraction()
    self.getTags()
    return Result(self.config, self.input_data.urlId)
  
  
  ## processTraining
  #
  def processConcurrency(self):
    #preamble
    resource_set = {}
    resource_set["url"] = self.input_data.url
    resource_set["resId"] = self.input_data.urlId
    resource_set["siteId"] = self.input_data.siteId
    resource_set["raw_html"] = self.input_data.raw_content
    
    result = {}
    resource = {}
    for extractor in self._extractors:
      resource[extractor.__class__.__name__] = Res(resource_set)
      result[extractor.__class__.__name__] = Result(self.config, self.input_data.urlId)
      extractor.extractTags(resource[extractor.__class__.__name__], result[extractor.__class__.__name__])
    
    response = Result(self.config, self.input_data.urlId)
    response.mergeResults(result)

    self.getProcessedContent(response)
    self.tagsCount = response.tagsCount
    self.tagsMask = response.tagsMask
    self.putArticleToDB(response)
    return response


  # #
  #
  #
  def getTags(self):
    #TODO: add processTraining method implementation
    #stub
    return None


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, query):
    self.logger.info("Connector: " + str(self.DBConnector))
    try:
      with closing(self.DBConnector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        self.DBConnector.commit()
        return cursor.fetchall()
    except mdb.Error as err:  # @todo logging in db_task
      self.DBConnector.rollback()
      self.logger.error("%s %s" % (err.args[0], err.args[1]))
      raise err


  ##
  #
  #
  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      dbHost = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_HOST)
      dbPort = int(self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PORT))
      dbUser = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_USER)
      dbPWD = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PWD)
      self.db_dc_scraper = self.config.get(className, CONSTS.DB_SCRAPERS)
      self.db_dc_scraper_db_connect = mdb.connect(dbHost, dbUser, dbPWD, self.db_dc_scraper, dbPort)
    except Exception, err:
      #tbi = Utils.getTracebackInfo()
      #self.logger.error(str(type(err)) + "\n")
      #self.logger.error(str(err.message) + "\n" + tbi)
      self.logger.debug(CONSTS.MSG_ERROR_LOAD_DB_BACKEND + str(err.message))


  ##
  #
  #
  def loadPropertiesFromDB(self):
    self.logger.debug("Start load Scraper's properties from DB")
    self.DBConnector = self.db_dc_scraper_db_connect
    try:
      query = "SELECT `Value` FROM %s WHERE `Name`='%s'" % (self.db_dc_scraper, self.__class__.__name__)
      self.logger.debug("query: %s" % query)
      res = self.executeQuery(query)
      if len(res)>0:
        self.properties = json.loads(res)
      else:
        self.logger.debug("properties wasn't found in %s table" % self.db_dc_scraper)
    except Exception, err:
      self.logger.debug("Error load Scraper's properties from DB: %s" % str(err.message))
      #stub
      #set properties manually
      #later it will be filled from db
      algorithm_dict = json.loads(CONSTS.SCRAPER_PROPERTIES_JSON)
      self.logger.debug("algorithm_dict: %s" % varDump(algorithm_dict))
      self.properties = algorithm_dict[CONSTS.PROPERTIES_KEY]

    self.logger.debug("Properties: %s" % varDump(self.properties))
    self.logger.debug("End load Scraper's properties from DB")


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def createModule(self, module_name):
    app = None
    try:
      app = (module_name, eval(module_name)(self.config))
      #app[1].setName(app[0])
      self.logger.debug("%s has been created!" % module_name)
    except Exception as err:
      self.logger.error("Exception has been thrown during %s creation. %s" % (module_name, err.message))
      tbi = Utils.getTracebackInfo()
      self.logger.error(str(vars(err)) + "\n" + tbi)
      raise
    return app[1]


  # #createApp
  # create application's pool
  #
  # @param app_name application name which instance will be created
  # @return instance of created application
  def getExtractorByName(self, extractor_name):
    for extractor in self._extractors:
      if extractor.__class__.__name__ == extractor_name:
        return extractor


  # #
  #
  #
  def getExitCode(self):
    return self.exit_code

  # #parseHost parse the root host name from url
  # for example: the result of http://s1.y1.example.com/path/to is example.com
  # @param url the full url
  # @return host of the url, eg: example.com
  @staticmethod
  def parseHost(url):
    host = None
    if urlparse(url).hostname:
      host = '.'.join(urlparse(url).hostname.split('.')[-2:])
    return host

