"""@package docstring
 @file base_extractor.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import logging
import copy
import re
import MySQLdb as mdb
import MySQLdb.cursors
from contextlib import closing
import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


# Local class constants
ERR_MSG_ADJUST_PUB_DATE = "Error in adjustPubDate: "
ERR_MSG_ADJUST_MEDIA = "Error in adjustMedia: "
ERR_MSG_ADJUST_CONTENT_UTF8_ENCODED = "Error in adjustContentUTF8Encoded: "

ERR_MSG_OK = ""

EMPTY_DATE = ""

# Adjust publication date
# If incoming publication date is array just return one of them (first)
# @param dates publication date, extracted from content
def adjustPubDate(dates):
  #logger.debug("dates: %s", dates)
  pub_date = EMPTY_DATE
  try:
    # TODO: improve to return most appropriate
    # if dates and any(i.isdigit() for i in dates):
    if isinstance(dates, list) and len(dates):
      # pub_date = dates[0]
      pub_date = " ".join(dates)
    else:
      pub_date = dates
    if pub_date and len(dates) and not re.search(r'\d+', pub_date):
      pub_date = EMPTY_DATE
  except Exception as err:
    logger.error(ERR_MSG_ADJUST_PUB_DATE + err.message)
  return pub_date


# Adjust data in media tag
# If media are PR (partial reference) adjust path
# @param medias media extracted from content
def adjustMedia(medias):
  return medias
  # valid_http_url = HttpUrl()
  # res = []
  # try:
  #  if isinstance(medias, list):
  #    for media in medias:
  #      if valid_http_url(media):
  #        res.append(media)
  # except Exception as err:
  #  logger.error(ERR_MSG_ADJUST_MEDIA + err.message)
  # return res


# Adjust data in content_encoded tag
# If content are non-meaningfull adjust it
# @param data content extracted from content
def adjustContentUTF8Encoded(data):
  return data


# Adjust data in content_encoded tag
# If content are non-meaningfull adjust it
# @param data content extracted from content
def adjustLink(data):
  if type(data) == list and len(data) > 1:
    data = data[0]
  return data



def adjustNone(data):
  return data


# #The BaseExtractor class
# This is the base class for custom extractors
# Provide basic functionality such as add tag, etc.
class BaseExtractor(object):


  tag = {
    CONSTS.TAG_MEDIA: adjustMedia,
    CONSTS.TAG_CONTENT_UTF8_ENCODED: adjustContentUTF8Encoded,
    CONSTS.TAG_PUB_DATE: adjustPubDate,
    CONSTS.TAG_TITLE: adjustNone,
    CONSTS.TAG_LINK: adjustLink,
    CONSTS.TAG_DESCRIPTION: adjustNone,
    CONSTS.TAG_DC_DATE: adjustNone,
    CONSTS.TAG_AUTHOR: adjustNone,
    CONSTS.TAG_GUID: adjustNone,
    CONSTS.TAG_KEYWORDS: adjustNone,
    CONSTS.TAG_MEDIA_THUMBNAIL: adjustNone,
    CONSTS.TAG_ENCLOSURE: adjustNone,
    CONSTS.TAG_MEDIA_CONTENT: adjustNone,
    CONSTS.TAG_GOOGLE: adjustNone,
    CONSTS.TAG_GOOGLE_TOTAL: adjustNone,
    CONSTS.HTML_LANG: adjustNone
  }


  tagsMask = {
    CONSTS.TAG_MEDIA: 0b1,
    CONSTS.TAG_CONTENT_UTF8_ENCODED: 0b10,
    CONSTS.TAG_PUB_DATE: 0b100,
    CONSTS.TAG_TITLE: 0b1000,

    CONSTS.TAG_LINK: 0b10000,
    CONSTS.TAG_DESCRIPTION: 0b100000,
    CONSTS.TAG_DC_DATE: 0b1000000,
    CONSTS.TAG_AUTHOR: 0b10000000,

    CONSTS.TAG_GUID: 0b100000000,
    CONSTS.TAG_KEYWORDS: 0b1000000000,
    CONSTS.TAG_MEDIA_THUMBNAIL: 0b10000000000,
    CONSTS.TAG_ENCLOSURE: 0b100000000000,

    CONSTS.TAG_MEDIA_CONTENT: 0b1000000000000,
    CONSTS.TAG_GOOGLE: 0b10000000000000,
    CONSTS.TAG_GOOGLE_TOTAL: 0b100000000000000,
    CONSTS.HTML_LANG: 0b1000000000000000
  }


  def __init__(self, config):
    #logger.debug("BASE CTOR START")
    self.db_dc_scraper_db_connect = None
    self.db_dc_scraper = None
    self.config = config
    
    #load MySQL db data
    self.loadDBBackend()

    self.name = "Base extractor"
    self.rank = CONSTS.SCRAPER_RANK_INIT

    #support processing modes
    self.process_mode = CONSTS.PROCESS_ALGORITHM_REGULAR
    self.modules= {}

    self.data = {"extractor":"Base extractor", "data":"", "name":""}
    self.loadPropertiesFromDB()
    #logger.debug("BASE CTOR END")

  def __str__(self):
    return "%s" % (self.name)


  def __repr__(self):
    return repr((self.name, self.rank))


  def isTagNotFilled(self, result, tag_name):
    #logger.debug("Result: <%s>", str(result.tags))
    #logger.debug("Tag type: <%s>", str(type(result.tags.get(tag_name))))
    #logger.debug("Tag value: <%s>", result.tags.get(tag_name))
    #logger.debug("Result's tags: %s", str(result.tags))
    if (isinstance(result.tags.get(tag_name),list) and len(result.tags.get(tag_name))) \
      or result.tags.get(tag_name) != "":
      #logger.debug("Tag : %s already filled", tag_name)
      pass
    else:
      #logger.debug("Tag : %s not filled", tag_name)
      pass
    return result.tags.get(tag_name) == ""


  def isTagValueNotEmpty(self, tag_value):
    full = None
    if type(tag_value) == list:
      # FIX IT: if list check if list items not empty
      #logger.debug("Tag data value: <%s>", tag_value)
      # Check if data in tag is empty list
      # return None if data list hold empty values
      # FIX IT: rewrite it in more convenient way
      if tag_value == ['']:
        #logger.debug("Tag data value is empty: <%s>", tag_value)
        full = None
      else:
        full = tag_value
    else:
      full = tag_value
    #logger.debug("Returned full value: <%s>", full)
    return full


  def addTag(self, result, tag_name, tag_value, xpath=""):
    ret = False
    try:
      # logger.info("result: " + str(result))
      # logger.info("tag_name: " + tag_name)
      if tag_value and not isinstance(tag_value, list):
        pass
        # logger.info("tag_value: " + tag_value)
      if tag_value and isinstance(tag_value, list):
        pass
        # :logger.info("tag_value: " + str(tag_value))
      # adjust tag values (e.g. pub_date, media, etc.)
      tag_value = self.tag[tag_name](tag_value)
    except Exception as err:
      logger.debug("Missing adjustment for section: " + err.message)
    # print "get: " , result.tags.get(tag_name)
    result.errorCode = 0
    result.errorMessage = ERR_MSG_OK

    #logger.debug("tag_name: %s", tag_name)
    #logger.debug("tag_value: %s", tag_value)
    #logger.debug("xpath: %s", xpath)
    if self.isTagNotFilled(result, tag_name) and self.isTagValueNotEmpty(tag_value):
    # if not result.tags.get(tag_name) and tag_value:
      # logger.debug("tag_name: %s, tag_value: %s", tag_name, tag_value)

      #self.data["data"] = tag_value
      #self.data["name"] = tag_name
      #self.data["xpath"] = xpath
      #result.tags[tag_name] = (copy.copy(self.data))
      data = {"extractor":"Base extractor", "data":"", "name":""}
      data["data"] = tag_value
      data["name"] = tag_name
      data["xpath"] = xpath
      data["extractor"] = self.__class__.__name__
      result.tags[tag_name] = data

      # result.tagsCount += 1
      # result.tagsMask = result.tagsMask | self.tagsMask[tag_name]
      ret = True
      # result.tags[tag_name]=self.data

      # print("tag_name: %s\n" %tag_name)
      # print("tag_value: %s\n" %tag_value)
      # print("self.data: %s\n" %self.data)
    return ret


  def loadDBBackend(self):
    try:
      className = self.__class__.__name__
      logger.debug("className: %s" % className)
      dbHost = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_HOST)
      dbPort = int(self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PORT))
      dbUser = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_USER)
      dbPWD = self.config.get(CONSTS.DB_SECTION, CONSTS.DB_PWD)
      self.db_dc_scraper = self.config.get(className, CONSTS.DB_SCRAPERS)
      self.db_dc_scraper_db_connect = mdb.connect(dbHost, dbUser, dbPWD, self.db_dc_scraper, dbPort)
    except Exception, err:
      logger.debug(CONSTS.MSG_ERROR_LOAD_DB_BACKEND + str(err.message))


  ##
  #
  #
  def loadPropertiesFromDB(self):
    logger.debug("Start load Scraper's properties from DB")
    self.DBConnector = self.db_dc_scraper_db_connect
    try:
      query = "SELECT `Value` FROM %s WHERE `Name`='%s'" % (self.db_dc_scraper, self.__class__.__name__)
      logger.debug("query: %s" % query)
      res = self.executeQuery(query)
      if len(res)>0:
        self.properties = json.loads(res)
      else:
        logger.debug("properties wasn't found in %s table" % self.db_dc_scraper)
    except Exception, err:
      logger.debug("Error load Scraper's properties from DB: %s" % str(err.message))
    logger.debug("End load Scraper's properties from DB")


  # #helper function for correct query execution
  #
  # @param query query for execution
  def executeQuery(self, query):
    logger.info("Connector: " + str(self.DBConnector))
    try:
      with closing(self.DBConnector.cursor(MySQLdb.cursors.DictCursor)) as cursor:
        cursor.execute(query)
        self.DBConnector.commit()
        return cursor.fetchall()
    except mdb.Error as err:  # @todo logging in db_task
      self.DBConnector.rollback()
      logger.error("%s %s" % (err.args[0], err.args[1]))
      raise err
