"""@package docstring
 @file newspaper_extractor.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


from base_extractor import BaseExtractor as bex
# from scraper_result import Result as Reslt
from newspaper import Article
# import json
import Constants as CONSTS
import app.Consts as APP_CONSTS
import logging
import json
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class NewspaperExtractor(bex):


  # local class constants general purpose
  EXTRACTOR_NAME = "Newspaper extractor"
  SECTION_NAME = "extractor"


  def __init__(self, config):
    bex.__init__(self, config)
    properties_json = '{"properties":{"rank":100,"tags":{"content_encoded":{"rank":1},"pubdate":{"rank":1},"media":{"rank":101},"description":{"rank":1},"author":{"rank":1},"keywords":{"rank":1},"title":{"rank":1},"link":{"rank":1}},"template":"{\\"a\\":\\"b\\"}"}}'
    properties_dict = json.loads(properties_json)#["properties"]
    logger.debug("properties_dict: %s" % varDump(properties_dict))
    self.properties = properties_dict[CONSTS.PROPERTIES_KEY]
    logger.debug("Properties: %s" % varDump(self.properties))
    
    
    if (CONSTS.RANK_KEY in self.properties):
      self.rank = self.properties[CONSTS.RANK_KEY]
      logger.debug("rank was setted: %s" % self.rank)
    self.name = "Newspaper extractor"
    self.data["extractor"] = "Newspaper extractor"


  def extractTags(self, resource, reslt):
    article = Article("")
    article.html = resource.raw_html
    # open("00.txt", "wb").write(resource.raw_html)
    article.is_downloaded = True
    try:
      article.parse()
      self.addTag(result=reslt, tag_name=CONSTS.TAG_TITLE, tag_value=article.title)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_LINK, tag_value=article.canonical_link)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_DESCRIPTION, tag_value=article.summary)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_PUB_DATE, tag_value=article.published_date)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_AUTHOR, tag_value=article.authors)
      # self.addTag(result=reslt, tag_name=CONSTS.TAG_GUID, tag_value=article.link_hash)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_DC_DATE, tag_value=article.additional_data)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, tag_value=article.text)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_KEYWORDS, tag_value=article.meta_keywords)
      self.addTag(result=reslt, tag_name=CONSTS.TAG_MEDIA, tag_value=article.imgs)
    except Exception as err:
      logger.warning("Newspaper parse error: " + err.message)
    return reslt
