"""@package docstring
 @file scraper_result.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import json
import time
import copy
import logging
from dc_processor.base_extractor import BaseExtractor
import dc_processor.Constants as CONSTS
import app.Consts as APP_CONSTS
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)


class Result(object):


  #name = "Scraper result object"
  #tags = {}
  #data = {}


  def __init__(self, config, resId):
    self.name = "Scraper result object"
    self.data = {}
    self.tags = {}
    self.resId = resId
    self.article_tags = [tag for tag in config.options("article_tags")]
    for tag in self.article_tags:
      self.addTag(tag)
    self.name = "Scraper result object"
    self.start = time.clock()
    self.errorCode = CONSTS.ERROR_OK
    self.errorMessage = CONSTS.MSG_ERROR_OK
    self.tagsCount = 0
    self.tagsMask = 0


  def getEmptyTags(self):
    empty_tags = [key for key, value in self.tags.items() if key in self.article_tags and not value]
    return empty_tags


  def getFilledTags(self):
    filled_tags = [key for key, value in self.tags.items() if key in self.article_tags and value]
    return filled_tags


  def addTag(self, tag):
    self.tags[tag] = ""


  def get(self):
    data = {}
    data["resId"] = self.resId
    data["tagList"] = []
    for key, value in self.tags.items():
      if value:
        # print "================"
        # print "key: %s | value: %s" % (key,value)
        tag = {}
        for key  in value:
          tag[key] = value[key]
          # print "tag %s" % tag
          # tag["name"] = value["name"]
          # tag["data"] = value["data"]
        self.tagsCount += 1
        self.tagsMask = self.tagsMask | BaseExtractor.tagsMask[tag["name"]]

        data["tagList"].append(copy.copy(tag))
    self.data["data"] = data
    self.data["error_code"] = self.errorCode
    self.data["error_message"] = self.errorMessage
    self.data["time"] = "%s sec." % (time.clock() - self.start)
    return json.dumps(self.data, sort_keys=True, indent=4, separators=(",", ":"))


  def mergeResults(self, results_dict):
    logging.debug("self: %s" % varDump(self))
    logging.debug("results_dict: %s" % varDump(results_dict))
    tmp = {}
    for item_name, item_value in self.tags.items():
      logging.debug("item_name: %s, item_value: %s" % (item_name, item_value))
      tmp[item_name] = []
      for key, value in results_dict.items():
        logging.debug("key: %s, value: %s" % (key, value))
        tmp[item_name].append(value.tags[item_name])

      logging.debug("BEFORE: %s" % varDump(tmp[item_name]))
      tmp[item_name] = self.getBestValue(tmp[item_name])
      logging.debug("AFTER: %s" % varDump(tmp[item_name]))
    logging.debug("SUMMARY: %s" % varDump(tmp))
    self.tags = tmp


  def getBestValue(self, items_list):
    tmp = [item for item in items_list if item!=""]
    response = ""
    # if more than one suggestions try to select best one
    if len(tmp)>1:
      # for each tag own rule
      # for content_encoded select biggest text
      if tmp[0]["name"]=="content_encoded":
        response = max(tmp, key = lambda x: x["data"])
      # for any else apply the same rule
      else:
        response = max(tmp, key = lambda x: x["data"])
    # if only one suggestion return it
    elif len(tmp)>0:
      response = tmp[0]
    # if no one suggestions return empty string
    else:
      response = ""
    return response
