"""@package docstring
 @file scrapy_extractor.py
 @author Alexey <developers.hce@gmail.com>
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""


import re
import ast
import json
import urllib
import logging
from urlparse import urljoin
from scrapy.selector import Selector
from dc_processor.base_extractor import BaseExtractor as bex
import dc_processor.Constants as CONSTS
import app.Utils as Utils  # pylint: disable=F0401
import app.Consts as APP_CONSTS
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

template = {}
google_template = {}


class ScrapyExtractor(bex):

  def __init__(self, config, templ=None):
    #logger.debug("DERIVED CTOR START")
    bex.__init__(self, config)

    #stub
    #set properties manually
    #later it will be filled from db
    #prepate algorithm dict
    properties_dict = json.loads(CONSTS.SCRAPY_PROPERTIES_JSON)
    logger.debug("properties_dict: %s" % varDump(properties_dict))
    self.properties = properties_dict[CONSTS.PROPERTIES_KEY]
    logger.debug("Properties: %s" % varDump(self.properties))
    
    # set module rank from module's properties
    if (CONSTS.RANK_KEY in self.properties):
      self.rank = self.properties[CONSTS.RANK_KEY]
      logger.debug("rank was setted: %s" % self.rank)
    
    # set use HTML5 from module's properties
    if (CONSTS.USE_HTML5_KEY in self.properties):
      self.use_html5 = self.properties[CONSTS.USE_HTML5_KEY]
    else:
      self.use_html5 = CONSTS.USE_HTML5_NO
    logger.debug("rank was setted: %s" % self.rank)

    self.name = "Scrapy extractor"
    self.data["extractor"] = "Scrapy extractor"
    self.sel = None
    self.resource = None
    template_file = config.get("Application", "template", None)
    if (CONSTS.TEMPLATE_KEY not in self.properties):
      self.template = json.loads(self.properties[CONSTS.TEMPLATE_KEY])
      logger.info("template: " + str(self.template))
    elif templ:
      self.template = json.loads(templ)
    elif template_file:
      try:
        logger.debug("Read template from file. %s", template_file)
        self.template = json.loads(open(template_file, "rb").read())
      except Exception, err:
        logger.error("Error Read template from file. %s" % (err.message))
    else:
      logger.error("Error Read template.")
    #logger.debug("DERIVED CTOR END")

  """
  ##
  #
  #
  def loadPropertiesFromDB(self):
    self.logger.debug("Start load Scraper's properties from DB")
    #stub
    #set properties manually
    #later it will be filled from db
    #prepate algorithm dict
    properties_json = '{"properties":{"rank":1,"tags":{"content_encoded":{"rank":1},"pubdate":{"rank":1},"media":{"rank":1},"description":{"rank":1},"author":{"rank":1},"keywords":{"rank":1},"title":{"rank":1},"link":{"rank":1}},"template":"{\"content_encoded\": [\"//div[contains(@class,\"Text\")]/p//text()\"]}"}}'
    properties_dict = json.loads(properties_json)
    self.logger.debug("properties_dict: %s" % varDump(properties_dict))
    self.properties = properties_dict[CONSTS.PROPERTIES_KEY]
    self.logger.debug("Properties: %s" % varDump(self.properties))
    self.logger.debug("End load Scraper's properties from DB")
  """

  def extractTags(self, resource, reslt):
    #logger.info("template: " + str(self.template))
    # print json.dumps(template, sort_keys=True, indent=4, separators=(",",":"))
    try:
      self.resource = resource
      logger.debug("URL: %s", self.resource.url)
      self.sel = Selector(text=resource.raw_html)

      # search engine parsing
      if resource.site_id=="d57f144e7b26c9976769ea94f18b9064":
        total, count = self.collectGoogleSearch()
        self.addTag(result=reslt, tag_name="google_search", tag_value=total)
        self.addTag(result=reslt, tag_name="google_search_total", tag_value=count)
        return reslt
      if resource.site_id=="1fe592caf03fd50c5f065c30f82b13bb":
        total, count = self.collectCabinetSearch()
        self.addTag(result=reslt, tag_name="google_search", tag_value=total)
        self.addTag(result=reslt, tag_name="google_search_total", tag_value=count)
        return reslt

      # reqular parsing
      else:
        # create result
        for path in self.template["title"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_TITLE, \
                     tag_value=[s for s in self.sel.xpath(path).extract()], xpath=path)
        for path in self.template["author"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_AUTHOR, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        for path in self.template["pubdate"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_PUB_DATE, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        for path in self.template["description"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_DESCRIPTION, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        for path in self.template["dc_date"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_DC_DATE, \
                     tag_value=(self.sel.xpath(path).extract()), xpath=path)
        for path in self.template["media_content"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_MEDIA, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)

        # [self.addTag(result=reslt, \
        # tag_name="media_thumbnail", \
        # tag_value=sel.xpath(path).extract()) \
        # for path in template["media_thumbnail"]]
        for path in self.template["link"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_LINK, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        if self.use_html5 == CONSTS.USE_HTML5_YES:
          import lxml
          text = self.sel.xpath("//body//*[not(self::script)]//article//text()").extract()
          #fragment  = lxml.html.fromstring(text[0].encode("utf-8"))
          #lxml.etree.strip_elements(fragment, 'script')
          #value = lxml.html.tostring(fragment)
          #import HTMLParser
          #h = HTMLParser.HTMLParser()
          #value = h.unescape(value)
          value = text
          self.addTag(result=reslt, \
                      tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, \
                      tag_value=value, xpath=path)
        else:
          for path in self.template["content_encoded"]:
            self.addTag(result=reslt, \
                       tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, \
                       tag_value=self.sel.xpath(path).extract(), xpath=path)
        for path in self.template["guid"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_GUID, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        # [self.addTag(result=reslt, \
        # tag_name="enclosure", \
        # tag_value=sel.xpath(path).extract()) \
        # for path in template["enclosure"]]
        for path in self.template["keywords"]:
          self.addTag(result=reslt, \
                     tag_name=CONSTS.TAG_KEYWORDS, \
                     tag_value=self.sel.xpath(path).extract(), xpath=path)
        for path in self.template["media"]:
            self.addTag(result=reslt, \
                        tag_name=CONSTS.TAG_MEDIA, \
                        tag_value=self.sel.xpath(path).extract(), xpath=path)

        # Add support of html_lang tag
        for path in self.template["html_lang"]:
            self.addTag(result=reslt, \
                        tag_name=CONSTS.HTML_LANG, \
                        tag_value=self.sel.xpath(path).extract(), xpath=path)
    except Exception as err:
      tbi = Utils.getTracebackInfo()
      logger.error(str(vars(err)) + "\n" + tbi)
      logger.error("Scrapy parse error: " + err.message)
    return reslt


  def collectGoogleSearch(self):
    # total count
    # for google.com
    # //*[@id="rso"]
    # //*[@id="rso"]/div/li[10]
    # for google.co.jp
    # for first page
    # //*[@id="resultStats"]/text()
    count = len(self.sel.xpath("//*[@id='ires']/ol/li").extract())
    self.logger.debug("results count: %s", str(count))
    total_results = self.sel.xpath("//*[@id='resultStats']/text()").extract()
    if len(total_results) > 0:
      text = total_results[0].split("bout")
      if len(text) > 1:
        text = text[1]
      else:
        text = total_results[0].split()[1]
      total_results = "".join(re.findall("\d+", text))
    self.logger.debug("total_results: %s", str(total_results))
    # num_elem = [1, 2]
    total = []
    for i in range(count):
      result = {}
      result["url"] = ""
      result["title"] = ""
      result["summary"] = ""
      # //*[@id="ires"]/ol/li[8]/table/tbody/tr[1]/td/h3/a/text()[1]
      summary = "//*[@id='ires']/ol/li[" + str(i + 1) + "]//h3/a//text()"
      # //*[@id="ires"]/ol/li[8]/table/tbody/tr[1]/td/h3/a
      link = "//*[@id='ires']/ol/li[" + str(i + 1) + "]//h3/a/@href"
      # //*[@id="rso"]/div/li[4]/div/div/div/span/em
      # //*[@id="rso"]/div/li[3]/div/div/div/span
      description = "//*[@id='ires']/ol/li[" + str(i + 1) + "]/div/span//text()"
      value = self.sel.xpath(link).extract()
      self.logger.debug("link: %s", str(value))
      if type(value) == list and len(value):
        value = value[0]
        self.logger.debug("value: %s", str(value))
        if len(value.split("?q=")) > 1:
          value = value.split("?q=")[1]
        # avoid & in url
        value = value.split("&")[0]
        # avoid double url encode
        value = urllib.unquote(value)
      result["url"] = value
      value = ''.join(self.sel.xpath(summary).extract())
      result["title"] = value
      value = ''.join(self.sel.xpath(description).extract())
      result["summary"] = value
      if not len(result["summary"]) or not len(result["title"]) or not len(result["url"]): continue
      total.append(result)
    return total, total_results


  def collectCabinetSearch(self):
    template_set = {}
    template_set["Action"] = {}
    template_set["Statements"] = {}
    template_set["Press"] = {}
    template_set["Diplomatic"] = {}

    template_set["Action"]["count"] = "//div[@class='wrap']//a/text()"
    template_set["Action"]["root"] = "//div[@class='wrap']"
    template_set["Action"]["title"] = "//a/text()"
    template_set["Action"]["link"] = "//a/@href"
    template_set["Action"]["description"] = "//p/text()"
    template_set["Action"]["thumbnail"] = "//img/@src"
    template_set["Action"]["pubdate"] = "//span/text()"
    
    template_set["Statements"]["count"] = "//div[@class='main-left png']/div[@class='body']/ul[@class='list-link']/li"
    template_set["Statements"]["root"] = "//div[@class='main-left png']/div[@class='body']/ul[@class='list-link']/li"
    template_set["Statements"]["title"] = "/a/text()"
    template_set["Statements"]["link"] = "/a/@href"
    template_set["Statements"]["description"] = "/a/text()"
    template_set["Statements"]["pubdate"] = "/text()"
    
    template_set["Press"]["count"] = "//table[@class='pressList']//td[@class='title']"
    template_set["Press"]["root"] = "(//table[@class='pressList']//td[@class='title'])"
    template_set["Press"]["title"] = "//a/text()"
    template_set["Press"]["link"] = "//a/@href"
    template_set["Press"]["description"] = "//a/text()"
    template_set["Press"]["pubdate"] = "/../..//th[@class='heading']/text()"
    
    template_set["Diplomatic"]["count"] = "//ul[@class='icolistA']/li"
    template_set["Diplomatic"]["root"] = "//ul[@class='icolistA']/li"
    template_set["Diplomatic"]["title"] = "//a/text()"
    template_set["Diplomatic"]["link"] = "//a/@href"
    template_set["Diplomatic"]["description"] = "//a/text()"
    template_set["Diplomatic"]["pubdate"] = "/text()"

    for key,value in template_set.items():
      keywords  = str(self.sel.xpath("//meta[@name='keywords']/@content").extract())
      if key in keywords:
        current_template = template_set[key]
        break
    self.logger.debug("Current template %s" % str(current_template))
    self.logger.debug("URL: %s" % self.resource.url)
    xpath = current_template["count"]
    self.logger.debug("XPath count: %s" % xpath)
    count = len(self.sel.xpath(xpath).extract())
    total_results = count
    self.logger.debug("results count: %s", str(count))
    total_results = count
    self.logger.debug("total_results: %s", str(total_results))
    total = []
    for i in range(count):
      result = {}
      result["url"] = ""
      result["title"] = ""
      result["summary"] = ""
      result["thumbnail"] = ""
      result["pubdate"] = ""
      
      title = current_template["root"] + "[" + str(i + 1) + "]" + current_template["title"]
      self.logger.debug("Title: %s" % title)
      
      link = current_template["root"] + "[" + str(i + 1) + "]" + current_template["link"]
      self.logger.debug("Link: %s" % link)
      
      description = current_template["root"] + "[" + str(i + 1) + "]" + current_template["description"]
      self.logger.debug("description: %s" % description)

      thumbnail = None
      if "thumbnail" in current_template:
        thumbnail = current_template["root"] + "[" + str(i + 1) + "]" + current_template["thumbnail"]
        self.logger.debug("thumbnail: %s" % thumbnail)
        value = self.sel.xpath(thumbnail).extract()
        self.logger.debug("thumbnail value: %s" % str(value))
        result["thumbnail"] = urljoin(self.resource.url, urllib.unquote(value[0]))

      pubdate = None
      if "pubdate" in current_template:
        pubdate = current_template["root"] + "[" + str(i + 1) + "]" + current_template["pubdate"]
        self.logger.debug("pubdate: %s" % pubdate)
        value = self.sel.xpath(pubdate).extract()
        self.logger.debug("pubdate value: %s" % str(value))
        result["pubdate"] = value[0].strip("\r\n\t[]")

      value = self.sel.xpath(link).extract()
      self.logger.debug("Link value: %s" % str(value))
      result["url"] = urljoin(self.resource.url, urllib.unquote(value[0]))

      value = ''.join(self.sel.xpath(title).extract())
      result["title"] = value

      value = ''.join(self.sel.xpath(description).extract())
      result["summary"] = value

      if result["thumbnail"]=="":
        del result["thumbnail"]
      if result["pubdate"]=="":
        del result["pubdate"]
      total.append(result)
    return total, total_results
