#!/bin/bash

CRITERION=$1
TASK_SOURCE_DIR=$2
OUTPUT_FILE=$3
DELIMITER=$4

printUsageAndExit()
## $1 - error message string
{
	printf "\nBad command line param. $1\nUsage: %s <task_id> <task_source_dir> <out_file_name> [<delimiter>]\n" $0
	echo "     <criterion> - criterion for search. You can use some criterions, if you will be use '$DELIMITER' as delimiter"
	echo "     <task_source_dir> - task log directory contents source files for search"
	echo "     <out_file_name> - out result file name"
	echo "     <delimiter> - delimiter for criterions use few words"
	exit 1
}

if [ -z "$CRITERION" ]
then
	printUsageAndExit "Mandatory must be first argument criterion for search."
fi

if [ -z "$TASK_SOURCE_DIR" ]
then
	printUsageAndExit "Mandatory must be second argument source directory for search."
fi

if [ -z "$OUTPUT_FILE" ]
then
	printUsageAndExit "Mandatory must be third argument output file name for save result of search."
fi

if [ -z "$DELIMITER" ]
then
  DELIMITER="|||"
fi

removeFile()
## $1 - file name for remove
{
	if [ -e "$1" ]
	then
		rm -f $1
	fi
}

searchTaskData()
## $1 - criterion for search
## $2 - task log directory
## $3 - out file name
{
	local criterions=() 
	OLDIFS=$IFS
	IFS="$DELIMITER" read -a criterions <<< "$CRITERION"
	IFS=$OLDIFS

	local sed_query="sed ";
	local grep_query="grep -lrw '"

#	echo "criterions: ${criterions[@]}" 
	a=0
	for one_criterion in "${criterions[@]}"
		do
			if [ ! -z $one_criterion ]
			then
				sed_query=("$sed_query""/$one_criterion/b;")

				if [ $a -ne 0 ]
				then
					grep_query=("$grep_query""\|") 
				fi
				a=$(($a+1))

				grep_query=("$grep_query""$one_criterion") 
			fi
		done

	sed_query=("$sed_query""d")

	grep_query=("$grep_query""' $TASK_SOURCE_DIR")

	log_files=(`eval $grep_query`) 

#	echo "found files: ${log_files[@]}"
#	echo "files count: ${#log_files[*]}"

	if [[ "${#log_files[*]}" -eq 0 ]]
	then
		echo "No found files with content: $CRITERION"
	else
		for log_file in "${log_files[@]}"
			do
				echo "$log_file content: " >> $OUTPUT_FILE
				echo " " >> $OUTPUT_FILE
				$sed_query $log_file >> $OUTPUT_FILE
				echo " " >> $OUTPUT_FILE
			done
	fi
}

removeFile "$OUTPUT_FILE"

searchTaskData "$CRITERION" "$TASK_SOURCE_DIR" "$OUTPUT_FILE"

exit 0

