/**
 * @file DataServerProxy.hpp
 *
 * HCE project DataServerProxy handler.
 * Implements functionality for data server proxy handler.
 * Contains client data requests server used in data and proxy nodes.
 *
 * @author bgv <bgv.hce@gmail.com>
 * @link http://hierarchical-cluster-engine.com/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://hierarchical-cluster-engine.com/license/
 * @package HCE project node API
 * @since 0.1
 */

#ifndef __HCE_DATASERVER_PROXY_INCLUDED__
#define __HCE_DATASERVER_PROXY_INCLUDED__

#include <sstream>
#include <map>

#include "Handler.hpp"

//Message cover object
#include "JsonMessageCover.hpp"

namespace HCE {
  namespace handlers {

    class DataServerProxy: public HCE::handlers::Handler {
      public:
        DataServerProxy(std::string name, zmq::context_t& ctx, const std::string& clientIdentity, std::string connectionString, unsigned char nodeMode, int64_t startedAt,
            unsigned char logPriority);
        ~DataServerProxy();

      protected:
        int initialize(void);
        void deinitialize(void);
        void* process(void);

        void handleInternalMessage(void);
        void handleExternalMessage(void);

        std::string processAdminCommands(std::string& command, std::string& parameters);

        void logPeriodicStatistics(void);
        void heartbit(void);
        //void heartbitResponse(const std::string& identity); //TODO: experimental_heartbitResponse
        void registerClient(const std::string& identity);

        void processControlMessage(zmsg& msg);
        void processDataMessage(zmsg& msg);

        enum class EmptyResponseType : std::int8_t
        {
          ertNoClients=0,
          ertBadNodeMode=1
        };
        std::string makeEmptyResponseMessage(const std::string& messageBody, EmptyResponseType responseType);

        //!<ZMQ Sockets
        zmq::socket_t* _inprocProxySock;
        zmq::socket_t* _inprocReducerInSock;
        zmq::socket_t* _dataServerSock;

        //!<Client ID
        std::string _clientIdentity;

        //!<Data node mode
        unsigned char _nodeMode;
        //!<Previous client number
        size_t _previousClientNumber;
        //!<Random client number genertor
        size_t getRandomNodeNumber(size_t maxNumber);


        //!<Queue of connected clients
        std::vector<ClientWorkerItem> _clientsQueue;
        ClientsQueueManager* _clientsManager;

        //!<Heartbeat at time mark
        int64_t _heartbeatAt;

        //!<Stat vars
        unsigned int _statClientRequests;
        unsigned int _statClientResponses;
        unsigned int _statRequests;
        unsigned int _statClientRequestsSize;
        unsigned int _statClientResponsesSize;
        unsigned int _statRequestsSize;
        unsigned int _periodicStatClientRequests;
        unsigned int _periodicStatClientResponses;
        unsigned int _statRequestsProcessed;
        unsigned int _statRequestsProcessedSize;
        unsigned int _periodicStatClientRequestsSize;
        unsigned int _periodicStatClientResponsesSize;
        float _statQPSClient;
        float _statQPSServer;
        float _statRPSClient;
        float _statRPSServer;
        float _statRequestsBandwidthClient;
        float _statRequestsBandwidthServer;
        float _statResponsesBandwidthClient;
        float _statResponsesBandwidthServer;

        //!<Send out periodical stats at regular intervals
        int64_t _pstatsAt = s_clock() + HEARTBEAT_INTERVAL;
    };
  }
}

#endif /* __HCE_DATASERVER_PROXY_INCLUDED__ */
