#include <sys/resource.h>
#include <sys/statvfs.h>
#include <utility>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fstream>
#include <sstream>
#include <Poco/TemporaryFile.h>
#include <Poco/Logger.h>
#include <Poco/Message.h>

#include "DRCEAsyncTasksQueue.hpp"
#include "DRCEResourceExtractor.hpp"
#include "DRCEMessageConst.hpp"
#include "DRCEError.hpp"
#include "DRCEReadProcessData.hpp"
#include "DRCEResourceUsage.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DiskSpaceExtractor::DiskSpaceExtractor(void)
: diskSize(0), diskUsed(0), diskFree(0), diskUsedPercent(0.0), diskFreePercent(0.0), externalDiskSize(0)
{
  try
  {
    update();
  }
  catch(Poco::Exception& e)
  {
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, "DiskSpaceExtractor "+e.message(), Poco::Message::Priority::PRIO_ERROR));
  }
}
//-----------------------------------------------------------------------------
void DiskSpaceExtractor::setDiskSize(size_t diskSize_)
{
  externalDiskSize = diskSize_;
}
//-----------------------------------------------------------------------------
void DiskSpaceExtractor::update(void) throw (Poco::Exception)
{
  Poco::TemporaryFile temporaryFile;
  temporaryFile.createFile();
  const std::string fileName = temporaryFile.path();

  struct statvfs64 buf={0};

  if (statvfs64(fileName.c_str(), &buf) != 0)
    throw Poco::Exception(std::string("DiskSpaceExtractor::update: ")+strerror(errno));

  size_t blksize = buf.f_bsize;
  size_t blocks = buf.f_blocks;
  size_t freeblks = buf.f_bfree;

  if (externalDiskSize)
    diskSize = externalDiskSize;
  else
    diskSize = blocks * blksize;

  diskFree = freeblks * blksize;
  diskUsed = diskSize - diskFree;

  diskUsedPercent = static_cast<double>(diskUsed*100)/diskSize;
  diskFreePercent = static_cast<double>(diskFree*100)/diskSize;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*CpuUsageExtractor::Pstat::Pstat(void)
: utimeTicks(0), cutimeTicks(0), stimeTicks(0), cstimeTicks(0), vsize(0), rss(0), cpuTotalTime(0)
{
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::Pstat::Pstat(const Pstat& rhs)
: utimeTicks(0), cutimeTicks(0), stimeTicks(0), cstimeTicks(0), vsize(0), rss(0), cpuTotalTime(0)
{
  *this = rhs;
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::Pstat::Pstat(Pstat&& rhs)
: utimeTicks(0), cutimeTicks(0), stimeTicks(0), cstimeTicks(0), vsize(0), rss(0), cpuTotalTime(0)
{
  *this = std::forward<Pstat>(rhs);
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::Pstat& CpuUsageExtractor::Pstat::operator=(const Pstat& rhs)
{
  if (this != &rhs)
  {
    utimeTicks = rhs.utimeTicks;
    cutimeTicks = rhs.cutimeTicks;
    stimeTicks = rhs.stimeTicks;
    cstimeTicks = rhs.cstimeTicks;
    vsize = rhs.vsize;
    rss = rhs.rss;
    cpuTotalTime = rhs.cpuTotalTime;
  }
  return (*this);
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::Pstat& CpuUsageExtractor::Pstat::operator=(Pstat&& rhs)
{
  if (this != &rhs)
  {
    utimeTicks = std::move(rhs.utimeTicks);
    cutimeTicks = std::move(rhs.cutimeTicks);
    stimeTicks = std::move(rhs.stimeTicks);
    cstimeTicks = std::move(rhs.cstimeTicks);
    vsize = std::move(rhs.vsize);
    rss = std::move(rhs.rss);
    cpuTotalTime = std::move(rhs.cpuTotalTime);
  }
  return (*this);
}
//-----------------------------------------------------------------------------
void CpuUsageExtractor::Pstat::clear(void)
{
  utimeTicks = 0;
  cutimeTicks = 0;
  stimeTicks = 0;
  cstimeTicks = 0;
  vsize = 0;
  rss = 0;
  cpuTotalTime = 0;
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::Pstat& CpuUsageExtractor::Pstat::operator+=(const Pstat& rhs)
{
  utimeTicks += rhs.utimeTicks;
  cutimeTicks += rhs.cutimeTicks;
  stimeTicks += rhs.stimeTicks;
  cstimeTicks += rhs.cstimeTicks;
  vsize += rhs.vsize;
  rss += rhs.rss;
  cpuTotalTime += rhs.cpuTotalTime;

  return *this;
}*/
//-----------------------------------------------------------------------------
/*
 * read /proc data into the passed struct pstat
 * returns 0 on success, -1 on error
*/
/*
int CpuUsageExtractor::getUsage(const pid_t pid, struct Pstat* result)
{
  std::stringstream statFileName;
  statFileName << "/proc/" << pid << "/stat";

  FILE *fpstat = fopen(statFileName.str().c_str(), "r");
  if (fpstat == NULL)
  {
    return -1;
  }

  FILE *uptstat = fopen("/proc/uptime", "r");
  if (uptstat == NULL)
  {
    fclose(uptstat);
    return -1;
  }


  //read values from /proc/pid/stat
  bzero(result, sizeof(struct Pstat));
  long int rss = 0;
  unsigned int starttime = 0;
  if (fscanf(fpstat, "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*u %*u %*u %*u %10lu"
              "%10lu %10ld %10ld %*d %*d %*d %*d %u %10lu %10ld",
              &result->utimeTicks, &result->stimeTicks,
              &result->cutimeTicks, &result->cstimeTicks, &starttime,
              &result->vsize, &rss) == EOF)
  {
//    fclose(fstat);
    fclose(uptstat);
    fclose(fpstat);
    return -1;
  }
  fclose(fpstat);
  result->rss = rss * getpagesize();
*/
/*
  //read+calc cpu total time from /proc/stat
  long unsigned int cpuTime[10];
  bzero(cpuTime, sizeof(cpuTime));
  if (fscanf(fstat, "%*s %10lu %10lu %10lu %10lu %10lu %10lu %10lu %10lu %10lu %10lu",
              &cpuTime[0], &cpuTime[1], &cpuTime[2], &cpuTime[3],
              &cpuTime[4], &cpuTime[5], &cpuTime[6], &cpuTime[7],
              &cpuTime[8], &cpuTime[9]) == EOF) {
    fclose(fstat);
    return -1;
  }

  fclose(fstat);

  for(int i=0; i < 10;i++)
      result->cpuTotalTime += cpuTime[i];
*//*
  double uptime = 0.0;
  if (fscanf(uptstat, "%*f %lf", &uptime) == EOF)
  {
    fclose(uptstat);
    return -1;
  }
  fclose(uptstat);

  result->cpuTotalTime = uptime - (starttime / sysconf(_SC_CLK_TCK));

  return 0;
}
*/
//-----------------------------------------------------------------------------
/*
* calculates the elapsed CPU usage between 2 measuring points. in percent
*/
/*
void CpuUsageExtractor::calcCpuUsagePct(const struct Pstat* curUsage,
                          const struct Pstat* lastUsage,
                          double* ucpuUsage, double* scpuUsage)
{
  double totalTimeDiff = (double)(curUsage->cpuTotalTime + lastUsage->cpuTotalTime)/2; //???
//  double totalTimeDiff = abs(curUsage->cpuTotalTime - lastUsage->cpuTotalTime);

//  std::cout << "totalTimeDiff: " << totalTimeDiff << " = " << curUsage->cpuTotalTime << " <-> " << lastUsage->cpuTotalTime << std::endl;

  if (totalTimeDiff)
  {
    *ucpuUsage = 100 * (double)(((curUsage->utimeTicks + curUsage->cutimeTicks)
                    - (lastUsage->utimeTicks + lastUsage->cutimeTicks))
                    / (double) totalTimeDiff);

    *scpuUsage = 100 * (double)((((curUsage->stimeTicks + curUsage->cstimeTicks)
                    - (lastUsage->stimeTicks + lastUsage->cstimeTicks)))
                    / (double) totalTimeDiff);
  }
*/
/*
  *ucpuUsage = 100 * (double)(curUsage->utimeTicks + curUsage->cutimeTicks)
//                      / (double)(curUsage->cpuTotalTime);
                      / sysconf(_SC_CLK_TCK);

  *scpuUsage = 100 * (double)(curUsage->stimeTicks + curUsage->cstimeTicks)
//                      / (double)(curUsage->cpuTotalTime);
                      / sysconf(_SC_CLK_TCK);
*//*
}
*/
//-----------------------------------------------------------------------------
/*
* calculates the elapsed CPU usage between 2 measuring points in ticks
*/
// cppcheck-suppress unusedFunction
/*
void CpuUsageExtractor::calcCpuUsage(const struct Pstat* curUsage,
                      const struct Pstat* lastUsage,
                      long unsigned int* ucpuUsage,
                      long unsigned int* scpuUsage)
{
  *ucpuUsage = (curUsage->utimeTicks + curUsage->cutimeTicks) -
                (lastUsage->utimeTicks + lastUsage->cutimeTicks);

  *scpuUsage = (curUsage->stimeTicks + curUsage->cstimeTicks) -
                (lastUsage->stimeTicks + lastUsage->cstimeTicks);
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void CpuUsageExtractor::getCpuData(pid_t pid, std::pair<unsigned long int, unsigned long int>& p)
{
  const unsigned int count = 14;
  char path[PATH_MAX]={0};
  char pId[PATH_MAX]={0};
  snprintf(path, PATH_MAX, "%s%d%s", "/proc/", pid, "/stat");

  FILE* fp = fopen(path, "r");
  if (fp)
  {
    for(unsigned int i = 1; i < count; ++i)
    {
      fscanf(fp, "%s", pId);
    }

    fscanf(fp, "%ld%ld", &p.first, &p.second);
    fclose(fp);
  }
}
//-----------------------------------------------------------------------------
/*
 * calculates the elapsed CPU usage for list of processes
 */
double CpuUsageExtractor::getCpuUsage(const std::vector<pid_t>& ids)
{
  const double defaultUsage = 100.0;
  const unsigned int SLPTIME = 200000;

  std::vector<std::pair<unsigned long int, unsigned long int> > cpuPrevData, cpuData;

  for (size_t i=0;i<ids.size();++i)
  {
    std::pair<unsigned long int, unsigned long int> data;
    getCpuData(ids[i], data);
    cpuPrevData.push_back(data);
  }

  usleep(SLPTIME);

  for (size_t i=0;i<ids.size();++i)
  {
    std::pair<unsigned long int, unsigned long int> data;
    getCpuData(ids[i], data);
    cpuData.push_back(data);
  }

  unsigned long int q(0), r(0);

  for (size_t i=0;i<cpuData.size();++i)
  {
    if ((cpuData[i].first) && (cpuPrevData[i].first))
      q += abs(cpuData[i].first-cpuPrevData[i].first);

    if ((cpuData[i].second) && (cpuPrevData[i].second))
      r += abs(cpuData[i].second-cpuPrevData[i].second);
  }

  double result = defaultUsage;
  long int frec = sysconf(_SC_CLK_TCK);
  if (frec > 0)
  {
    result = ((q + r)*100.0/((double)0.2*frec));
  }
  return result;
}
//-----------------------------------------------------------------------------
CpuUsageExtractor::CpuUsageExtractor(void)
: cpuUsage(0.0), lastBusy(0), lastWork(0), iowait(0.0), lastIOWait(0), lastSum(0)
{
  const unsigned int SLPTIME = 200000;
  update();
  usleep(SLPTIME);
  update();
}
//-----------------------------------------------------------------------------
void CpuUsageExtractor::update(void)
{
  unsigned long long cpu = 0;
  unsigned long long nice = 0;
  unsigned long long system = 0;
  unsigned long long idle = 0;

  unsigned long long ioWait = 0;
  unsigned long long irq = 0;
  unsigned long long sofrirq = 0;
  unsigned long long steal = 0;
  unsigned long long guest = 0;

  std::ifstream ifs("/proc/stat");
  if (ifs.is_open())
  {
    std::string line;
    while(getline(ifs, line))
    {
      if (line.find("cpu")!=std::string::npos)
      {
        sscanf(line.c_str(), "cpu %20llu %20llu %20llu %20llu %20llu %20llu %20llu %20llu %20llu", &cpu, &nice, &system, &idle, &ioWait, &irq, &sofrirq, &steal, &guest);
        break;
      }
    }
    ifs.close();
  }
  unsigned long long busy = cpu + nice + system;
  unsigned long long work = busy + idle;

  if (work!=lastWork)
    cpuUsage = static_cast<double>(100*(busy-lastBusy))/(work-lastWork);
  lastBusy = busy;
  lastWork = work;

  unsigned long long sum = (cpu+nice+system+idle+ioWait+irq+sofrirq+steal+guest);

  if (sum!=lastSum)
    iowait = (ioWait-lastIOWait)*100/(sum-lastSum);

  lastIOWait = ioWait;
  lastSum = sum;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
MemoryExtractor::MemInfo::MemInfo(void)
: memTotal(0), memFree(0), swapTotal(0), swapFree(0), vmallocTotal(0), vmallocUsed(0)
{
}
//-----------------------------------------------------------------------------
MemoryExtractor::MemInfo::MemInfo(MemInfo&& rhs)
: memTotal(0), memFree(0), swapTotal(0), swapFree(0), vmallocTotal(0), vmallocUsed(0)
{
  *this = std::forward<MemInfo>(rhs);
}
//-----------------------------------------------------------------------------
MemoryExtractor::MemInfo& MemoryExtractor::MemInfo::operator=(MemInfo&& rhs)
{
  if (this != &rhs)
  {
    memTotal = std::move(rhs.memTotal);
    memFree = std::move(rhs.memFree);
    swapTotal = std::move(rhs.swapTotal);
    swapFree = std::move(rhs.swapFree);
    vmallocTotal = std::move(rhs.vmallocTotal);
    vmallocUsed = std::move(rhs.vmallocUsed);
  }
  return *this;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
MemoryExtractor::MemoryExtractor(void)
: vramUsed(0), rramUsed(0), vramFree(0), rramFree(0), vramUsedPercent(0.0), rramUsedPercent(0.0), vramFreePercent(0.0), rramFreePercent(0.0),
  externalVramTotal(0), externalRramTotal(0)
{
  update();
}
//-----------------------------------------------------------------------------
void MemoryExtractor::setVramTotal(size_t vramTotal_)
{
  externalVramTotal = vramTotal_;
}
//-----------------------------------------------------------------------------
void MemoryExtractor::setRramTotal(size_t rramTotal_)
{
  externalRramTotal = rramTotal_;
}
//-----------------------------------------------------------------------------
void MemoryExtractor::update(void)
{
  MemoryExtractor::MemInfo memInfo = getMemoryInfo();

  if (externalRramTotal)
    memInfo.memTotal = externalRramTotal;

  rramFree = memInfo.memFree*1024;
  rramUsed = (memInfo.memTotal-memInfo.memFree)*1024;

  size_t vmemTotal = (externalVramTotal)?externalVramTotal:(memInfo.swapTotal+memInfo.memTotal);
  vramFree = (vmemTotal-memInfo.vmallocUsed)*1024;
  vramUsed = memInfo.vmallocUsed*1024;

  rramFreePercent = static_cast<double>(rramFree*100)/(memInfo.memTotal*1024);
  rramUsedPercent = static_cast<double>(rramUsed*100)/(memInfo.memTotal*1024);

  vramFreePercent = static_cast<double>(vramFree*100)/(vmemTotal*1024);
  vramUsedPercent = static_cast<double>(vramUsed*100)/(vmemTotal*1024);

  vramTotal = vmemTotal*1024;
  rramTotal = memInfo.memTotal*1024;
}
//-----------------------------------------------------------------------------
MemoryExtractor::MemInfo MemoryExtractor::getMemoryInfo(void)
{
  MemoryExtractor::MemInfo memInfo;

  std::ifstream ifs("/proc/meminfo");
  if (ifs.is_open())
  {
    std::string line;
    while(getline(ifs, line))
    {
      if (line.find("MemTotal")!=std::string::npos)
        sscanf(line.c_str(), "MemTotal: %20zu", &(memInfo.memTotal));

      if (line.find("MemFree")!=std::string::npos)
        sscanf(line.c_str(), "MemFree: %20zu", &(memInfo.memFree));

      if (line.find("SwapTotal")!=std::string::npos)
        sscanf(line.c_str(), "SwapTotal: %20zu", &(memInfo.swapTotal));

      if (line.find("SwapFree")!=std::string::npos)
        sscanf(line.c_str(), "SwapFree: %20zu", &(memInfo.swapFree));

      if (line.find("VmallocTotal")!=std::string::npos)
        sscanf(line.c_str(), "VmallocTotal: %20zu", &(memInfo.vmallocTotal));

      if (line.find("VmallocUsed")!=std::string::npos)
        sscanf(line.c_str(), "VmallocUsed: %20zu", &(memInfo.vmallocUsed));
    }
    ifs.close();
  }
  return memInfo;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray::FieldsArray(void)
: cpu(0.0), vram(0), rram(0), threads(0)
{
}
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray::FieldsArray(const FieldsArray& rhs)
: cpu(0.0), vram(0), rram(0), threads(0)
{
  *this = rhs;
}
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray::FieldsArray(FieldsArray&& rhs)
: cpu(0.0), vram(0), rram(0), threads(0)
{
  *this = std::forward<FieldsArray>(rhs);
}
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray& ProcessInfoExtractor::FieldsArray::operator=(const FieldsArray& rhs)
{
  if (this != &rhs)
  {
    cpu = rhs.cpu;
    vram = rhs.vram;
    rram = rhs.rram;
    threads = rhs.threads;
  }
  return *this;
}
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray& ProcessInfoExtractor::FieldsArray::operator=(FieldsArray&& rhs)
{
  if (this != &rhs)
  {
    cpu = std::move(rhs.cpu);
    vram = std::move(rhs.vram);
    rram = std::move(rhs.rram);
    threads = std::move(rhs.threads);
  }
  return *this;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ProcessInfoExtractor::FieldsArray ProcessInfoExtractor::extract(AsyncTasks& asyncTask)
{
  std::vector<pid_t> ids = DRCEReadProcessData::getAllProcessIds(asyncTask.pid);

  ProcessInfoExtractor::FieldsArray fieldsArray;

  for (size_t i=0;i<ids.size();++i)
  {
    ProcessData processData = DRCEReadProcessData::getInstance().getProcessData(ids[i]);

    //!<Accumulate information about use memory
    fieldsArray.vram += processData.vram;
    fieldsArray.rram += processData.rram;

    //!<Accumulate information about use thread count
    fieldsArray.threads += processData.threads;

    //!<Accumulate information about CPU usage
    fieldsArray.cpu = CpuUsageExtractor::getCpuUsage(ids);
  }
  return fieldsArray;
}
//-----------------------------------------------------------------------------
ResourceUsageExtractor* ResourceUsageExtractor::pHandler = nullptr;
Poco::Mutex ResourceUsageExtractor::mutex;
//-----------------------------------------------------------------------------
ResourceUsageExtractor::ResourceUsageExtractor(void)
: cpuUsageExtractor(), diskSpaceExtractor(), memoryExtractor(), maxThreadsCount(0), maxProcessesCount(0)
{
}
//-----------------------------------------------------------------------------
ResourceUsageExtractor& ResourceUsageExtractor::getInstance(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  if(!pHandler)
    pHandler = new ResourceUsageExtractor();
  return *pHandler;
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::shutdown(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  if(pHandler)
  {
    delete pHandler;
    pHandler=nullptr;
  }
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::update(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  cpuUsageExtractor.update();
  diskSpaceExtractor.update();
  memoryExtractor.update();
}
//-----------------------------------------------------------------------------
ResourceUsage ResourceUsageExtractor::extract(void) throw (Poco::Exception)
{
  Poco::Mutex::ScopedLock lock(mutex);
  ResourceUsage resourceUsage;

  resourceUsage.cpu = cpuUsageExtractor.getCpuUsage();
  resourceUsage.iowait = cpuUsageExtractor.getIOWait();

  resourceUsage.diskUsed = diskSpaceExtractor.getDiskUsed();
  resourceUsage.diskUsedPercent = diskSpaceExtractor.getDiskUsedPercent();

  resourceUsage.vramUsed = memoryExtractor.getVramUsed();
  resourceUsage.vramUsedPercent = memoryExtractor.getVramUsedPercent();
  resourceUsage.rramUsed = memoryExtractor.getRramUsed();
  resourceUsage.rramUsedPercent = memoryExtractor.getRramUsedPercent();

  DRCEReadProcessData::getInstance().getProcessAndThreads(resourceUsage.processes, resourceUsage.threads);

  long sz = sysconf(_SC_PAGESIZE);
  if (sz<0)
    throw Poco::Exception(std::string("ResourceUsageExtractor::extract: ")+strerror(errno));

//  printf("Page size = %ld bytes \n", sz);

  size_t stacksize = 0;
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_getstacksize(&attr, &stacksize);
//  printf("Thread stack size = %zu bytes \n", stacksize);
  stacksize /= 1024;
//  printf("Thread stack size = %zu Kb \n", stacksize);

  size_t totalRamPages = (memoryExtractor.getRramTotal()/sz);
//  printf("Total ram pages = %zu\n", totalRamPages);

  size_t maxThreads = (maxThreadsCount)?maxThreadsCount:(totalRamPages/(8*stacksize/sz));
//  printf("Max threads = %zu\n", maxThreads);

//  std::cout << "getMaxProcesses(): " << getMaxProcesses() << std::endl;
//  std::cout << "getMaxThreads(): " << getMaxThreads() << std::endl;

  resourceUsage.threadsPercent = static_cast<double>(resourceUsage.threads*100)/maxThreads;

  double numberThreads = static_cast<double>(memoryExtractor.getVramTotal())/(stacksize*1024*1024*1024);
//  printf("Number of threads = %f\n", numberThreads);

  double maxProcesses = (maxProcessesCount)?maxProcessesCount:(static_cast<double>(maxThreads) / numberThreads);
//  printf("Max Processes = %f\n", maxProcesses);

  resourceUsage.processesPercent = static_cast<double>(resourceUsage.processes*100)/maxProcesses;

//  struct rlimit rlim={0};
//  getrlimit(RLIMIT_NPROC, &rlim);
//  std::cout << "rlim.rlim_cur = " << rlim.rlim_cur << " rlim.rlim_max = " << rlim.rlim_max << std::endl;

  //TODO in future, here can be necessary refactoring
  return resourceUsage;
}
//-----------------------------------------------------------------------------
CpuUsageExtractor& ResourceUsageExtractor::getCpuUsageExtractor(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return cpuUsageExtractor;
}
//-----------------------------------------------------------------------------
DiskSpaceExtractor& ResourceUsageExtractor::getDiskSpaceExtractor(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return diskSpaceExtractor;
}
//-----------------------------------------------------------------------------
MemoryExtractor& ResourceUsageExtractor::getMemoryExtractor(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return memoryExtractor;
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::setMaxThreadsCount(size_t maxThreadsCount_)
{
  Poco::Mutex::ScopedLock lock(mutex);
  maxThreadsCount=maxThreadsCount_;
}
//-----------------------------------------------------------------------------
size_t ResourceUsageExtractor::getMaxThreadsCount(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return maxThreadsCount;
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::setMaxProcessesCount(size_t maxProcessesCount_)
{
  Poco::Mutex::ScopedLock lock(mutex);
  maxProcessesCount=maxProcessesCount_;
}
//-----------------------------------------------------------------------------
size_t ResourceUsageExtractor::getMaxProcessesCount(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return maxProcessesCount;
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::setMaxDiskSize(size_t maxDiskSize_)
{
  Poco::Mutex::ScopedLock lock(mutex);
  diskSpaceExtractor.setDiskSize(maxDiskSize_);
}
//-----------------------------------------------------------------------------
size_t ResourceUsageExtractor::getMaxDiskSize(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return diskSpaceExtractor.getDiskSize();
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::setMaxVramSize(size_t maxVramSize_)
{
  Poco::Mutex::ScopedLock lock(mutex);
  memoryExtractor.setVramTotal(maxVramSize_);
}
//-----------------------------------------------------------------------------
size_t ResourceUsageExtractor::getMaxVramSize(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return memoryExtractor.getVramTotal();
}
//-----------------------------------------------------------------------------
void ResourceUsageExtractor::setMaxRramSize(size_t maxRramSize_)
{
  Poco::Mutex::ScopedLock lock(mutex);
  memoryExtractor.setRramTotal(maxRramSize_);
}
//-----------------------------------------------------------------------------
size_t ResourceUsageExtractor::getMaxRramSize(void)
{
  Poco::Mutex::ScopedLock lock(mutex);
  return memoryExtractor.getRramTotal();
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
unsigned int ResourceUsageExtractor::getMaxProcesses(void)
{
  const std::string fileName= "/proc/sys/kernel/pid_max";
  return extractValue(fileName);
}
//-----------------------------------------------------------------------------
unsigned int ResourceUsageExtractor::getMaxThreads(void)
{
  const std::string fileName= "/proc/sys/kernel/threads-max";
  return extractValue(fileName);
}
//-----------------------------------------------------------------------------
unsigned int ResourceUsageExtractor::extractValue(const std::string& fileName)
{
  unsigned int value = 0;
  std::ifstream ifs(fileName.c_str());
  if (ifs.is_open())
  {
    ifs >> value;
    ifs.close();
  }
  return value;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
