#include <utility>

#include "DRCEResourceLimits.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(void)
: procMax(0), threadsMax(0), cpu(0.0), iowait(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0)
{
}
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(const ResourceLimits& rhs)
: procMax(0), threadsMax(0), cpu(0.0), iowait(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0)
{
  (*this) = rhs;
}
//-----------------------------------------------------------------------------
ResourceLimits::ResourceLimits(ResourceLimits&& rhs)
: procMax(0), threadsMax(0), cpu(0.0), iowait(0.0), vramFree(0.0), vramFreeMin(0),
  rramFree(0.0), rramFreeMin(0), diskFree(0.0), diskFreeMin(0)
{
  (*this) = std::forward<ResourceLimits>(rhs);
}
//-----------------------------------------------------------------------------
ResourceLimits& ResourceLimits::operator=(const ResourceLimits& rhs)
{
  if (this!=&rhs)
  {
    procMax = rhs.procMax;
    threadsMax = rhs.threadsMax;
    cpu = rhs.cpu;
    iowait = rhs.iowait;
    vramFree = rhs.vramFree;
    vramFreeMin = rhs.vramFreeMin;
    rramFree = rhs.rramFree;
    rramFreeMin = rhs.rramFreeMin;
    diskFree = rhs.diskFree;
    diskFreeMin = rhs.diskFreeMin;
  }
  return *this;
}
//-----------------------------------------------------------------------------
ResourceLimits& ResourceLimits::operator=(ResourceLimits&& rhs)
{
  if (this!=&rhs)
  {
    procMax = std::move(rhs.procMax);
    threadsMax = std::move(rhs.threadsMax);
    cpu = std::move(rhs.cpu);
    iowait = std::move(rhs.iowait);
    vramFree = std::move(rhs.vramFree);
    vramFreeMin = std::move(rhs.vramFreeMin);
    rramFree = std::move(rhs.rramFree);
    rramFreeMin = std::move(rhs.rramFreeMin);
    diskFree = std::move(rhs.diskFree);
    diskFreeMin = std::move(rhs.diskFreeMin);
  }
  return *this;
}
//-----------------------------------------------------------------------------
void ResourceLimits::clear(void)
{
  procMax = 0;
  threadsMax = 0;
  cpu = 0.0;
  iowait = 0.0;
  vramFree = 0.0;
  vramFreeMin = 0;
  rramFree = 0.0;
  rramFreeMin = 0;
  diskFree = 0.0;
  diskFreeMin = 0;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
