#include <signal.h>
#include <sys/wait.h>
#include <Poco/Timestamp.h>
#include <Poco/Logger.h>
#include <Poco/File.h>

#include "Process.hpp"
#include "DRCETaskRequestTerminateExecutor.hpp"
#include "DRCETaskRequestTerminate.hpp"
#include "DRCEFileExtractor.hpp"
#include "DRCEFileBuilder.hpp"
#include "DRCEFileStream.hpp"
#include "DRCECleanupTask.hpp"
#include "DRCECommonTask.hpp"
#include "DRCEReadProcessData.hpp"

namespace HCE
{
namespace drce
{
//-----------------------------------------------------------------------------
DRCETaskRequestTerminateExecutor::DRCETaskRequestTerminateExecutor(DRCEAsyncTasksQueue& asyncTaskQueue_,
    DRCENodeOptions& nodeOptions_, CustomMessage& message_, DRCEResourceMonitor& resourceMonitor_)
: inherited(asyncTaskQueue_, nodeOptions_, message_, resourceMonitor_)
{
}
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestTerminateExecutor::execute(DRCETaskRequest* pTaskRequest, DRCEInputJsonMessage& inputJsonMessage)
{
  DRCEResultDataItem resultDataItem = getResultDataItem(*pTaskRequest, DRCETaskRequest::TaskState::UNDEFINED);

  try
  {
    DRCETaskRequestTerminate* pTaskRequestTerminate = dynamic_cast<DRCETaskRequestTerminate*>(pTaskRequest);
    if (!pTaskRequestTerminate)
      throw Poco::Exception(message(message_const::WRONG_TYPE_TASK_REQUEST), ERROR_BAD_COMMAND_PARAMS);

    if (pTaskRequestTerminate->getDelayValue() == 0 || pTaskRequestTerminate->getRepeatValue() == 0)
      throw Poco::Exception(message(message_const::BAD_COMMAND_PARAMS), ERROR_BAD_COMMAND_PARAMS);

    resultDataItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::NOT_FOUND));

    if (!asyncTaskQueue.isExistAsyncTask(pTaskRequestTerminate->getTaskId()))
    {
      if (!loadStatusFile(resultDataItem))
        throw Poco::Exception(message(message_const::TASK_NOT_FOUND), ERROR_TERMINATE_TASK);
    }
    //!<Save result data on disk
/*    resultDataItem =  DRCECommonTask::makeResultDataItem(inputJsonMessage, nodeOptions, message, DRCETaskRequest::RequestType::rtTerminateTask,
          DRCETaskRequest::TaskState::TERMINATED, ERROR_TERMINATE_SUBTASK_AFTER_EXPIRED_MAIN_TASK, message(message_const::TASK_TIME_EXPIRED),
          0, 0, resultDataItem.getExitStatus());
    resultDataItem.setErrorCode(ERROR_TERMINATE_EXPIRED_TASK);
    DRCECommonTask::saveResultData(resultDataItem, nodeOptions, message,  asyncTaskQueue);
///?    resultDataItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::UNDEFINED));
*/
    std::stringstream outMsg;
    outMsg << "Request terminate taskId: " << pTaskRequestTerminate->getTaskId();
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_TRACE));

    DRCECommonTask::SubtasksList subtasks;
    DRCEInputJsonMessage inputJsonMessage = DRCECommonTask::getRequestData(pTaskRequestTerminate->getTaskId(), nodeOptions, message);
    DRCECommonTask::getSubtasksList(inputJsonMessage, subtasks);

    // below block code need to checking
    for (DRCECommonTask::SubtasksList::reverse_iterator iter=subtasks.rbegin();iter!=subtasks.rend();++iter)
    {
      std::cout << "TASK ID: " << (*iter).first << "  isCleanup: " << std::boolalpha << (pTaskRequestTerminate->getCleanupFlag()==DRCETaskRequestTerminate::CleanupFlag::cfDelete) << std::endl;

//////      applyCleanup((*iter).first);

      asyncTaskQueue.cancelTask((*iter).first, (pTaskRequestTerminate->getCleanupFlag()==DRCETaskRequestTerminate::CleanupFlag::cfDelete));
      terminateTask((*iter).first, *pTaskRequestTerminate, resultDataItem);

/****/
      Poco::File statusFile(FileStream::getStatusFileName(nodeOptions.getTasksStatusDir(), (*iter).first));
      if (statusFile.exists())
      {
        unsigned int errorCode = NO_ERROR;
        unsigned int errorCodeMainTask = NO_ERROR;
        std::string errorMessage;

        DRCEResultDataItem resultItem = DRCECommonTask::extractStatusResultDataItem((*iter).first, nodeOptions, asyncTaskQueue);
        if (resultItem.getErrorCode() == ERROR_TERMINATE_EXPIRED_TASK)
        {
          errorCode = ERROR_TERMINATE_SUBTASK_AFTER_EXPIRED_MAIN_TASK;
          errorCodeMainTask = ERROR_TERMINATE_EXPIRED_TASK;
          errorMessage = message(message_const::TASK_TIME_EXPIRED);
        }

        resultDataItem = DRCECommonTask::makeResultDataItem(inputJsonMessage, nodeOptions, message,
            DRCETaskRequest::RequestType::rtSetTaskExecute, DRCETaskRequest::TaskState::TERMINATED,
            errorCode, errorMessage, 0,0, resultDataItem.getExitStatus());
        resultDataItem.setErrorCode(errorCodeMainTask);

 //       if (asyncTaskQueue.isExistAsynTask((*iter).first))
          DRCECommonTask::saveResultData(resultDataItem, nodeOptions, message, asyncTaskQueue);
      }
/****/

//      DRCECommonTask::saveRequest(inputJsonMessage, nodeOptions, message);
//      DRCECommonTask::saveResultData(resultDataItem, nodeOptions, message, asyncTaskQueue);
    }

//    DRCECommonTask::waitEndAllTask(inputJsonMessage, asyncTaskQueue);

//    resultDataItem = makeResultDataItem(inputJsonMessage, resultDataItem.getExitStatus());

//    if (pTaskRequestTerminate->getCleanupFlag() != DRCETaskRequestTerminate::CleanupFlag::cfDelete)
//    {
//      DRCECommonTask::saveRequest(inputJsonMessage, nodeOptions, message);
//      DRCECommonTask::saveResultData(resultDataItem, nodeOptions, message, asyncTaskQueue);
//    }
/********************/
//    DRCECommonTask::waitEndAllTask(inputJsonMessage, asyncTaskQueue);
//--remove here    DRCECommonTask::waitUpdateAllTasks(pTaskRequestTerminate->getTaskId(), nodeOptions, message, asyncTaskQueue);
//    DRCECommonTask::cleanup(inputJsonMessage, nodeOptions, (pTaskRequestTerminate->getCleanupFlag()==DRCETaskRequestTerminate::CleanupFlag::cfDelete));
/*
    DRCECommonTask::saveResultData(resultDataItem, nodeOptions, message, asyncTaskQueue);

    if (pTaskRequestTerminate->getCleanupFlag()==DRCETaskRequestTerminate::CleanupFlag::cfDelete)
      prepareCleanup(pTaskRequestTerminate->getTaskId(), inputJsonMessage);

    DRCECommonTask::saveRequest(inputJsonMessage, nodeOptions, message);

    outMsg.str("");
    outMsg << "subtasks for cleanup: ";
    for (DRCECommonTask::SubtasksList::iterator iter=subtasks.begin();iter!=subtasks.end();++iter)
    {
      outMsg << (*iter).first << " ";
    }
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_TRACE));

    unsigned int taskId = DRCECommonTask::getHostParentTaskId(pTaskRequestTerminate->getTaskId(), asyncTaskQueue);
    outMsg.str("");
    outMsg << "hostParentTaskId: " << taskId;
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_TRACE));

    while (asyncTaskQueue.isExistAsyncTask(taskId))
    {
      Poco::Thread::sleep(1000);
      outMsg.str("");
      outMsg << "DRCETaskRequestTerminateExecutor::execute  asyncTaskQueue.isExistAsyncTask(" << taskId << ")";
      Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));

      AsyncTasks asyncTasks = asyncTaskQueue.getAsyncTask(taskId);
      if (Process::isExistProcess(asyncTasks.pid) && asyncTasks.pid)
      {
        kill(asyncTasks.pid, SIGKILL);
        outMsg.str("");
        outMsg << "DRCETaskRequestTerminateExecutor::execute  force kill (" << asyncTasks.pid << ")";
        Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));
      }
      else
      {
        bool res = asyncTaskQueue.removeAsyncTask(taskId);
        outMsg.str("");
        outMsg << "DRCETaskRequestTerminateExecutor::execute  Erase ( " << taskId << " ) from async task queue return: " << std::boolalpha << res;
        Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));
      }
    }

    DRCECommonTask::cleanup(inputJsonMessage, nodeOptions, (pTaskRequestTerminate->getCleanupFlag()==DRCETaskRequestTerminate::CleanupFlag::cfDelete));
*/
  }
  catch(Poco::Exception& e)
  {
    resultDataItem.setErrorCode(e.code());
    resultDataItem.setErrorMessage(e.message());
    AsyncTasks asyncTask(asyncTaskQueue.getAsyncTask(pTaskRequest->getTaskId()));
//    setTaskState(std::forward<AsyncTasks>(AsyncTasks(asyncTask)), resultDataItem, DRCETaskRequest::TaskState::CRASHED);

    asyncTask.timeElapsed = DRCECommonTask::getElapsedTimeMsec(Poco::Timestamp::fromEpochTime(asyncTask.timeStart));
    asyncTask.state = DRCETaskRequest::TaskState::CRASHED;
    if (asyncTaskQueue.isExistAsyncTask(pTaskRequest->getTaskId()))
      asyncTaskQueue.safeChangeState(std::forward<AsyncTasks>(asyncTask), &resultDataItem, e.message(), e.code());
  }
  catch(std::exception& e)
  {
    resultDataItem.setErrorCode(ERROR_EXECUTE_TASK);
    resultDataItem.setErrorMessage(e.what());
    AsyncTasks asyncTask(asyncTaskQueue.getAsyncTask(pTaskRequest->getTaskId()));
 //   setTaskState(std::forward<AsyncTasks>(AsyncTasks(asyncTask)), resultDataItem, DRCETaskRequest::TaskState::CRASHED);
    asyncTask.timeElapsed = DRCECommonTask::getElapsedTimeMsec(Poco::Timestamp::fromEpochTime(asyncTask.timeStart));
    asyncTask.state = DRCETaskRequest::TaskState::CRASHED;
    if (asyncTaskQueue.isExistAsyncTask(pTaskRequest->getTaskId()))
      asyncTaskQueue.safeChangeState(std::forward<AsyncTasks>(asyncTask), &resultDataItem, std::string(e.what()), ERROR_EXECUTE_TASK);
  }
  return resultDataItem;
}
//-----------------------------------------------------------------------------
void DRCETaskRequestTerminateExecutor::executeDefaultAlgorithm(DRCETaskRequestTerminate& taskRequestTerminate, pid_t pid,
    DRCEResultDataItem& resultDataItem) throw (Poco::Exception)
{
  if (pid)
  {
    kill(pid, SIGTERM);
    resultDataItem.setExitStatus(HCE::drce::drce_const::EXIT_STATUS_BASE+SIGTERM);
    if (HCE::Process::isExistProcess(pid))
    {
      Poco::Thread::sleep(taskRequestTerminate.getDelayValue());

      std::string errorMsg;
      for (unsigned int i=0;i<taskRequestTerminate.getRepeatValue();++i)
      {
        if (kill(pid, SIGKILL) != 0)
          errorMsg = strerror(errno);
        if (HCE::Process::isExistProcess(pid))
          Poco::Thread::sleep(taskRequestTerminate.getDelayValue());
        else
          break;
      }
      if (!errorMsg.empty())
      {
        if (!loadStatusFile(resultDataItem))
          throw Poco::Exception(message(message_const::PROCESS_WAS_NOT_STOPPED, errorMsg)+
                        message(message_const::PID, pid), ERROR_TERMINATE_TASK);
      }
      resultDataItem.setExitStatus(HCE::drce::drce_const::EXIT_STATUS_BASE+SIGKILL);
    }
  }
}
//-----------------------------------------------------------------------------
void DRCETaskRequestTerminateExecutor::executeCustomAlgorithm(DRCETaskRequestTerminate& taskRequestTerminate, pid_t pid,
    DRCEResultDataItem& resultDataItem) throw (Poco::Exception)
{
  if (pid)
  {
    std::string errorMsg;
    for (unsigned int i=0;i<taskRequestTerminate.getRepeatValue();++i)
    {
      if (kill(pid, taskRequestTerminate.getSignalValue()) != 0)
        errorMsg = strerror(errno);
      if (HCE::Process::isExistProcess(pid))
        Poco::Thread::sleep(taskRequestTerminate.getDelayValue());
      else
        break;
    }
    if (!errorMsg.empty())
    {
      if (!loadStatusFile(resultDataItem))
        throw Poco::Exception(message(message_const::PROCESS_WAS_NOT_STOPPED, errorMsg)+
                      message(message_const::PID, pid), ERROR_TERMINATE_TASK);
    }
    resultDataItem.setExitStatus(HCE::drce::drce_const::EXIT_STATUS_BASE+taskRequestTerminate.getSignalValue());
  }
}
//-----------------------------------------------------------------------------
/*****
void DRCETaskRequestTerminateExecutor::setTaskState(AsyncTasks&& asyncTask, DRCEResultDataItem& resultDataItem, DRCETaskRequest::TaskState taskState)
{
  asyncTask.state = taskState;
  if (asyncTask.timeStart)
  {
    asyncTask.timeElapsed = DRCECommonTask::getElapsedTimeMsec(Poco::Timestamp::fromEpochTime(asyncTask.timeStart));
    resultDataItem.setTime(asyncTask.timeElapsed);
  }
  resultDataItem.setState(static_cast<unsigned int>(taskState));

  if (asyncTaskQueue.isExistAsyncTask(asyncTask.taskId))
    asyncTaskQueue.safeChangeState(std::forward<AsyncTasks>(asyncTask), nullptr);
}
*****/
//-----------------------------------------------------------------------------
bool DRCETaskRequestTerminateExecutor::loadStatusFile(DRCEResultDataItem& resultDataItem)
{
  bool result = false;
  Poco::File statusFile(FileStream::getStatusFileName(nodeOptions.getTasksStatusDir(), resultDataItem.getRequestId()));
  if (statusFile.exists())
  {
    try
    {
      AsyncTaskLocker lock(asyncTaskQueue, resultDataItem.getRequestId());
      DataFileExtractor dataFileExtractor(message);
      resultDataItem = dataFileExtractor.extract(statusFile.path());
      result = true;
    }
    catch(Poco::Exception& e)
    {
      Poco::Logger::root().log(Poco::Message(drce_const::moduleName, e.displayText(), Poco::Message::Priority::PRIO_TRACE));
    }
  }
  return result;
}
//-----------------------------------------------------------------------------
void DRCETaskRequestTerminateExecutor::terminateTask(unsigned int taskId,
    DRCETaskRequestTerminate& taskRequestTerminate,
    DRCEResultDataItem& resultDataItem) throw (Poco::Exception)
{
  if (asyncTaskQueue.isExistAsyncTask(taskId))
  {
    AsyncTasks asyncTask(asyncTaskQueue.getAsyncTask(taskId));
    resultDataItem.setRequestId(taskId);
    resultDataItem.setPid(asyncTask.pid);

    std::vector<pid_t> descendants = DRCEReadProcessData::getAllProcessIds(asyncTask.pid);
    switch(taskRequestTerminate.getAlgorithmType())
    {
      case DRCETaskRequestTerminate::AlgorithmType::atDefault:
        {
          executeDefaultAlgorithm(taskRequestTerminate, asyncTask.pid, resultDataItem);
        }
        break;
      case DRCETaskRequestTerminate::AlgorithmType::atCustom:
        {
          if (taskRequestTerminate.getSignalValue() == 0)
            throw Poco::Exception(message(message_const::BAD_COMMAND_PARAMS), ERROR_BAD_COMMAND_PARAMS);
          executeCustomAlgorithm(taskRequestTerminate, asyncTask.pid, resultDataItem);
        }
        break;
      default: throw Poco::Exception(message(message_const::WRONG_ALGORITHM_TYPE,
          static_cast<unsigned int>(taskRequestTerminate.getAlgorithmType())), ERROR_BAD_COMMAND_PARAMS);
    }

    std::stringstream outMsg;
    outMsg << "descendants count = " << descendants.size() << " (";
    for (size_t i=0;i<descendants.size();++i)
    {
      if (i!=0)
        outMsg << ", ";
      outMsg << descendants[i];
      kill(descendants[i], SIGKILL);
    }
    outMsg << ") has been killed...";
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_TRACE));


///    setTaskState(std::forward<AsyncTasks>(AsyncTasks(asyncTask)), resultDataItem, DRCETaskRequest::TaskState::TERMINATED);
    asyncTask.timeElapsed = DRCECommonTask::getElapsedTimeMsec(Poco::Timestamp::fromEpochTime(asyncTask.timeStart));
    asyncTask.state = DRCETaskRequest::TaskState::TERMINATED;
//    if (asyncTaskQueue.isExistAsyncTask(taskId))
      asyncTaskQueue.safeChangeState(std::forward<AsyncTasks>(asyncTask), &resultDataItem);
  }
  else
  {
    std::stringstream outMsg;
    outMsg << "Task for terminate [ " << taskId << " ] alredy not exist in async queue";
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));
  }
}
//-----------------------------------------------------------------------------
/*void DRCETaskRequestTerminateExecutor::applyCleanup(unsigned int taskId)
{
  DRCEInputJsonMessage inputJsonMessage = DRCECommonTask::getRequestData(taskId, nodeOptions, message);
  DRCETaskRequestSetExecute* pTaskRequestSetExecute = dynamic_cast<DRCETaskRequestSetExecute*>(inputJsonMessage.getTaskRequest().get());

  std::cout << "pTaskRequestSetExecute = " << (bool)(pTaskRequestSetExecute) << std::endl;
  if (pTaskRequestSetExecute)
  {
    HCE::drce::SessionOptions sessionOptions = pTaskRequestSetExecute->getSessionOptions();
    sessionOptions.cleanup = HCE::drce::SessionOptions::CleanupFlag::cfDelete;
    pTaskRequestSetExecute->setSessionOptions(std::forward<HCE::drce::SessionOptions>(sessionOptions));
    inputJsonMessage.setTaskRequest(pTaskRequestSetExecute);
    inputJsonMessage.set
    DRCECommonTask::saveRequest(inputJsonMessage, nodeOptions, message);
  }
}*/
//-----------------------------------------------------------------------------
DRCEResultDataItem DRCETaskRequestTerminateExecutor::makeResultDataItem(DRCEInputJsonMessage& inputJsonMessage, unsigned int exitStatus)
{
  return DRCECommonTask::makeResultDataItem(inputJsonMessage,
      nodeOptions,
      message,
      DRCETaskRequest::RequestType::rtTerminateTask,
      DRCETaskRequest::TaskState::TERMINATED,
      NO_ERROR,
      "",
      0,
      0, //
      exitStatus);
}
//-----------------------------------------------------------------------------
/*
void DRCETaskRequestTerminateExecutor::updateResultDataItem(DRCEResultDataItem& resultDataItem)
{
  const std::string statusFileName = FileStream::getStatusFileName(nodeOptions.getTasksStatusDir(), resultDataItem.getRequestId());
  Poco::File statusFile(statusFileName);
  if (statusFile.exists())
  {
    DRCEResultDataItem resultItem;
    resultItem.setRequestId(resultDataItem.getRequestId());
    if (loadStatusFile(resultItem))
    {
      resultDataItem.setPid(resultItem.getPid());
      resultDataItem.setTime(resultItem.getTime());
    }
  }
  resultDataItem.setState(static_cast<unsigned int>(DRCETaskRequest::TaskState::TERMINATED));

  for (size_t i=0;i<resultDataItem.getSubtasksCount();++i)
  {
    DRCEResultDataItem resultItem(resultDataItem.getSubtaskItem(i));
    updateResultDataItem(resultItem);
    resultDataItem.setSubtaskItem(i, std::forward<DRCEResultDataItem>(resultItem));
  }

  try
  {
    AsyncTaskLocker lock(asyncTaskQueue, resultDataItem.getRequestId());
    DRCEResultData resultData;
    resultData.addDataItem(resultDataItem);
    DataFileBuilder dataFileBuilder(message, nodeOptions.getTasksDataDir());
    dataFileBuilder.build(resultData);

    StatusFileBuilder statusFileBuilder(message, nodeOptions.getTasksStatusDir());
    statusFileBuilder.build(resultData);
  }
  catch(Poco::Exception& e)
  {
    std::stringstream outMsg;
    outMsg << "updateResultDataItem " << e.displayText();
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));
  }
  catch(std::exception& e)
  {
    std::stringstream outMsg;
    outMsg << "updateResultDataItem " << e.what();
    Poco::Logger::root().log(Poco::Message(drce_const::moduleName, outMsg.str(), Poco::Message::Priority::PRIO_DEBUG));
  }
}
*/
//-----------------------------------------------------------------------------
/*
void DRCETaskRequestTerminateExecutor::prepareCleanup(unsigned int taskId, DRCEInputJsonMessage& inputJsonMessage)
{
  if (taskId == inputJsonMessage.getRequestId())
  {
    Poco::SharedPtr<DRCETaskRequest> pTaskRequest = inputJsonMessage.getTaskRequest();
    DRCETaskRequestSetExecute* pTaskRequestSetExecute = dynamic_cast<DRCETaskRequestSetExecute*>(pTaskRequest.get());
    if (pTaskRequestSetExecute)
    {
      SessionOptions sessionOptions = pTaskRequestSetExecute->getSessionOptions();
      sessionOptions.cleanup = SessionOptions::CleanupFlag::cfDelete;
      pTaskRequestSetExecute->setSessionOptions(std::forward<SessionOptions>(sessionOptions));
      pTaskRequest = pTaskRequestSetExecute;
      inputJsonMessage.setTaskRequest(pTaskRequest);
    }
  }
  else
  {
    for (size_t i=0;i<inputJsonMessage.getSubtasksCount();++i)
    {
      DRCEInputJsonMessage inputMessage(inputJsonMessage.getSubtaskItem(i));
      prepareCleanup(taskId, inputMessage);
    }
  }
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
} // end namespace drce 
} // end namespace HCE 
