#include <gtest/gtest.h>

#include "DRCEProtocolsTest.hpp"
#include "DRCEFunctionalObjectTest.hpp"
#include "DRCEActionTypesTest.hpp"
#include "DRCERequestLocalShellTest.hpp"
#include "DRCETaskThreadingModeTest.hpp"
#include "DRCETaskRequestsTest.hpp"
#include "DRCECleanupTest.hpp"
#include "DRCEMessageConst.hpp"
#include "DRCEReadProcessData.hpp" //

#include <Poco/Channel.h>
#include <Poco/ConsoleChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/FormattingChannel.h>
#include <Poco/PatternFormatter.h>
#include <Poco/Logger.h>
#include <Poco/AutoPtr.h>

#include <random>
#include <map>

#include "DRCEResourceUsage.hpp"

#define TEST_DEBUG 1

namespace HCE
{
namespace drce
{
namespace tests
{
//-----------------------------------------------------------------------------
void loggerInitialization(const std::string& loggerName, Poco::AutoPtr<Poco::Channel> pCustomerChannel, const std::string& level)
{
  Poco::AutoPtr<Poco::PatternFormatter> pPatternFormatter(new Poco::PatternFormatter("%Y-%m-%d %H:%M:%S.%i - %N[%P] - %s - %q - %t"));
  if (pPatternFormatter)
  {
    Poco::AutoPtr<Poco::FormattingChannel> pChannel = new Poco::FormattingChannel(pPatternFormatter);
    if (pChannel)
    {
      Poco::AutoPtr<Poco::Channel> pCustomer(pCustomerChannel);
      pChannel->setChannel(pCustomer);
      pChannel->open();
      Poco::Logger::get(loggerName).setChannel(pChannel);
      Poco::Logger::get(loggerName).setLevel(level);
    }
  }
}
//-----------------------------------------------------------------------------
} // end namespace tests
} // end namespace drce
} // end namespace HCE

#if TEST_DEBUG == 0

int main(int argc, char** argv)
{
  // loggers intialization
  HCE::drce::tests::loggerInitialization(Poco::Logger::ROOT, new Poco::ConsoleChannel(), "trace");

  Poco::AutoPtr<Poco::FileChannel> pFileChannel = new Poco::FileChannel();
  pFileChannel->setProperty("path", "./tasks.log");
  HCE::drce::tests::loggerInitialization(HCE::drce::drce_const::DRCE_TASKS_LOGGER, pFileChannel, "information");
/*
  HCE::drce::tests::DRCEProtocolsTest::testResultDataSerializator();
  HCE::drce::tests::DRCEProtocolsTest::testInputJsonMessage();
  HCE::drce::tests::DRCEProtocolsTest::testOutputJsonMessage();

  HCE::drce::tests::DRCEProtocolsTest::testSessionOptions();
  HCE::drce::tests::DRCEProtocolsTest::testFilesItemList();

  HCE::drce::tests::DRCEActionTypesTest::testCreateBeforeExec();
  HCE::drce::tests::DRCEActionTypesTest::testDeleteBeforeExec();
  HCE::drce::tests::DRCEActionTypesTest::testReadAfterExec();
  HCE::drce::tests::DRCEActionTypesTest::testDeleteAfterExec();
  HCE::drce::tests::DRCEActionTypesTest::testEncodeDecodeContent();

  HCE::drce::tests::DRCERequestLocalShellTest::testEmptyCommand();
  HCE::drce::tests::DRCERequestLocalShellTest::testInputStreamWithCommand();

  HCE::drce::tests::DRCERequestLocalShellTest::testInputStreamWithoutCommand();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandSort();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandLs();
  HCE::drce::tests::DRCERequestLocalShellTest::testBadCommandLs();

  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPs();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPhp();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandPerl();
  HCE::drce::tests::DRCERequestLocalShellTest::testCommandJava();

  HCE::drce::tests::DRCEFunctionalObjectTest::testApplyEnviroments();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSortInput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSortOutput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellSort();

  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPsInput();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPsOutput();

  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPs();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPhpVersion();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPhpScript();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPerlVersion();
  HCE::drce::tests::DRCEFunctionalObjectTest::testRequestLocalShellPerlScript();

  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeSetTaskToExecuteRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeCheckTaskRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeGetTaskDataRequest();
  HCE::drce::tests::DRCETaskRequestsTest::testSerializeUnserializeTerminateTaskRequest();

  HCE::drce::tests::DRCETaskThreadingModeTest::testTerminateTaskRequestCleanUp();
  HCE::drce::tests::DRCETaskThreadingModeTest::testRequestLocalShellAsyncCleanUp();

  HCE::drce::tests::DRCETaskThreadingModeTest::testRequestLocalShellAsync();
  HCE::drce::tests::DRCETaskThreadingModeTest::testStressMultipleRequestsShellHost();

//  HCE::drce::tests::DRCETaskThreadingModeTest::testResourceMonitor(); // resource monitor to stdout
  HCE::drce::tests::DRCETaskThreadingModeTest::testCurrentTasksQueue();
  HCE::drce::tests::DRCETaskThreadingModeTest::testListAllTasks();
*//*
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskAsyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskSyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskAsyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskSyncSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskMixSubtasks();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskMixSubtasks();
  HCE::drce::tests::DRCECleanupTest::testAsyncTaskMixSubtasksCleanup();
  HCE::drce::tests::DRCECleanupTest::testSyncTaskMixSubtasksCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestGetDataCleanup();
  HCE::drce::tests::DRCECleanupTest::testRequestDeleteDataCleanup();*/
  HCE::drce::tests::DRCECleanupTest::testRequestTerminateCleanup();
//  HCE::drce::tests::DRCECleanupTest::testRequestDeleteData();

/*
  std::random_device rd;
  std::mt19937 gen(rd());
  std::uniform_real_distribution<double> dis(0, 3);

  std::map<int, size_t> Map;

  for(size_t i = 0;i<10000;++i)
  {
    double uniformOn01 = dis(gen);
    std::map<int, size_t>::iterator iter = Map.find(static_cast<int>(uniformOn01));
    if (iter!=Map.end())
    {
      ++((*iter).second);
    }
    else
    {
      Map[static_cast<int>(uniformOn01)] = 1;
    }
  }

  size_t index = 0;
  for (std::map<int, size_t>::iterator iter=Map.begin();iter!=Map.end();++iter)
  {
    std::cout << index++ << ") [ " << (*iter).first << " ] = " << (*iter).second << std::endl;
  }
*/
  /*
  HCE::drce::ResourceUsageExtractor::getInstance().setMaxThreadsCount(1024);
  HCE::drce::ResourceUsageExtractor::getInstance().setMaxProcessesCount(1024);

  for (size_t i=0;i<3;++i)
  {
    HCE::drce::ResourceUsageExtractor::getInstance().update();
    HCE::drce::ResourceUsage resourceUsage = HCE::drce::ResourceUsageExtractor::getInstance().extract();
    std::cout << "===============================================" << std::endl;
    std::cout << "resourceUsage.cpu:              " << resourceUsage.cpu << std::endl;
    std::cout << "resourceUsage.iowait:           " << resourceUsage.iowait << std::endl << std::endl;
    std::cout << "resourceUsage.diskUsed:         " << resourceUsage.diskUsed << std::endl;
    std::cout << "resourceUsage.diskUsedPercent:  " << resourceUsage.diskUsedPercent << std::endl << std::endl;
    std::cout << "resourceUsage.vramUsed:         " << resourceUsage.vramUsed << std::endl;
    std::cout << "resourceUsage.vramUsedPercent:  " << resourceUsage.vramUsedPercent << std::endl << std::endl;
    std::cout << "resourceUsage.rramUsed:         " << resourceUsage.rramUsed << std::endl;
    std::cout << "resourceUsage.rramUsedPercent:  " << resourceUsage.rramUsedPercent << std::endl << std::endl;
    std::cout << "resourceUsage.processes:        " << resourceUsage.processes << std::endl;
    std::cout << "resourceUsage.processesPercent: " << resourceUsage.processesPercent << std::endl << std::endl;
    std::cout << "resourceUsage.threads:          " << resourceUsage.threads << std::endl;
    std::cout << "resourceUsage.threadsPercent:   " << resourceUsage.threadsPercent << std::endl;
    usleep(2000000);
  }

  HCE::drce::ResourceUsageExtractor::shutdown();
  HCE::drce::DRCEReadProcessData::shutdown();
*/
/*
  double d = 23.123456789;
  double r = 0.0;

  std::cout << "d = " << d << std::endl;

  std::stringstream str;
  str.precision(3);
  str << d;
  str >> r;

  std::cout << "r = " << r << std::endl;
*/
/*
  double d = 23.123456789;
  int n = (int)d;

  double t = d-n;
  double t2 = 0.0;

  std::stringstream str2;
  str2.precision(3);
  str2 << t;
  str2 >> t2;
  t2 += n;

  std::cout << "t2 = " << t2 << std::endl;
*/
  std::cout << "Done\n";

  return 0;
}
#else
  //  svn://192.168.1.201/hce/hce_dts/trunk/api/php/bin/drce_fetch_resources.sh

int main (int argc, char** argv)
{
  // loggers intialization
  HCE::drce::tests::loggerInitialization(Poco::Logger::ROOT, new Poco::ConsoleChannel(), "information");
  Poco::AutoPtr<Poco::FileChannel> pFileChannel = new Poco::FileChannel();
  pFileChannel->setProperty("path", "./tasks.log");
  HCE::drce::tests::loggerInitialization(HCE::drce::drce_const::DRCE_TASKS_LOGGER, pFileChannel, "information");

  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}

#endif // end TEST_DEBUG
